'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

const index = require('./index-8af2820a.js');
const windowResizeService = require('./window-resize-service-65a5a370.js');
const focusTrap_esm = require('./focus-trap.esm-7bc384a0.js');
require('./utils-f8ccab0a.js');
require('./BehaviorSubject-12116537.js');

const contentNavigationBlockCss = "*{box-sizing:border-box}body.content-navigation-block-open{overflow:hidden}content-navigation-block button{appearance:none;border:none;background:transparent;cursor:pointer}content-navigation-block .content-navigation-block__skip-content a,content-navigation-block .content-navigation-block__skip-content button,content-navigation-block .content-navigation-block__skip-content a:visited,content-navigation-block .content-navigation-block__skip-content button:visited{border:2px solid rgb(0, 0, 0);border-radius:2px;color:purple;cursor:pointer;display:inline-block;font-size:14px;line-height:20px;margin:6px 11px;min-height:32px;text-decoration:underline;text-align:center;width:106px;z-index:1000}content-navigation-block .content-navigation-block__skip-content:not(:focus-within){position:absolute;clip:rect(1px, 1px, 1px, 1px);overflow:hidden;padding:0}content-navigation-block svg.default-icon{display:inline-block;width:32px}content-navigation-block[hamburger=fixed],content-navigation-block[hamburger=absolute],content-navigation-block[hamburger=static]{display:inline-block}content-navigation-block[hamburger=fixed][aria-expanded=false] .content-navigation__close,content-navigation-block[hamburger=absolute][aria-expanded=false] .content-navigation__close,content-navigation-block[hamburger=static][aria-expanded=false] .content-navigation__close{display:none}content-navigation-block[hamburger=fixed] .content-navigation__open,content-navigation-block[hamburger=fixed] .content-navigation__close,content-navigation-block[hamburger=absolute] .content-navigation__open,content-navigation-block[hamburger=absolute] .content-navigation__close,content-navigation-block[hamburger=static] .content-navigation__open,content-navigation-block[hamburger=static] .content-navigation__close{cursor:pointer}content-navigation-block[hamburger=fixed] .content-navigation__content-wrapper,content-navigation-block[hamburger=absolute] .content-navigation__content-wrapper,content-navigation-block[hamburger=static] .content-navigation__content-wrapper{display:none}content-navigation-block[hamburger=fixed] .content-navigation__content-wrapper[hidden=false],content-navigation-block[hamburger=absolute] .content-navigation__content-wrapper[hidden=false],content-navigation-block[hamburger=static] .content-navigation__content-wrapper[hidden=false]{display:block}content-navigation-block[hamburger=fixed] .content-navigation__content-wrapper{position:fixed;left:0;top:0;width:100vw;height:100vh;z-index:100}content-navigation-block[hamburger=sticky] .content-navigation__content-wrapper{position:sticky;left:0;top:0;width:100vw;height:100vh;z-index:100}content-navigation-block[hamburger=absolute] .content-navigation__content-wrapper{position:absolute;z-index:100}content-navigation-block[hamburger=static] .content-navigation__content-wrapper{position:static}content-navigation-block[hamburger=none]{display:block}content-navigation-block[hamburger=none] .content-navigation__open,content-navigation-block[hamburger=none] .content-navigation__close{display:none}content-navigation-block[aria-expanded=true] .content-navigation__content-wrapper{display:block}";
const ContentNavigationBlockStyle0 = contentNavigationBlockCss;

const ContentNavigationBlock = class {
    constructor(hostRef) {
        index.registerInstance(this, hostRef);
        this.hamburger = 'fixed|none';
        this.classes = undefined;
        this.classString = undefined;
        this.ariaExpanded = 'false';
        this.bodyToggle = false;
        this.skipAccessibilityMessage = undefined;
        this.hasOpenButton = false;
        this.hasCloseButton = false;
        this.openButton = undefined;
        this.closeButton = undefined;
        this.contentSlot = undefined;
    }
    componentWillLoad() {
        this.hasOpenButton = !!this.hostElement.querySelector('[slot="openButton"]');
        this.hasCloseButton = !!this.hostElement.querySelector('[slot="closeButton"]');
        windowResizeService.WindowResizeService.registerProp(this.hamburger).subscribe(newValue => {
            this.hamburger = newValue;
            this.bodyToggle = this.hamburger === 'fixed';
            document.body.classList.remove('content-navigation-block-open');
        });
        windowResizeService.WindowResizeService.registerProp(this.classes).subscribe(newValue => {
            this.classString = newValue;
        });
    }
    componentDidLoad() {
        this.openButton.onclick = () => {
            this.openMenu();
        };
        this.closeButton.onclick = () => {
            this.closeMenu();
        };
    }
    openMenu() {
        this.ariaExpanded = this.ariaExpanded === 'true' ? 'false' : 'true';
        if (this.ariaExpanded === 'false') {
            document.body.classList.remove('content-navigation-block-open');
        }
        else if (this.bodyToggle) {
            document.body.classList.add('content-navigation-block-open');
        }
        this.focusTrap = focusTrap_esm.createFocusTrap(this.contentSlot.parentElement, {
            initialFocus: this.contentSlot,
            fallbackFocus: this.hostElement,
            clickOutsideDeactivates: true,
            onDeactivate: () => {
                this.closeMenu();
                this.focusTrap = null;
                this.ariaExpanded = 'false';
                document.body.classList.remove('content-navigation-block-open');
                setTimeout(() => this.openButton.focus(), 1);
            },
        });
        this.focusTrap.activate();
    }
    closeMenu() {
        if (this.focusTrap) {
            try {
                this.focusTrap.deactivate();
            }
            catch (e) {
                console.log(e);
            }
        }
    }
    render() {
        return (index.h(index.Host, { key: '60e3048ab56cc17e7522ce3bccc527c4bed19d84', class: this.classString, "aria-expanded": this.ariaExpanded }, index.h("div", { key: '17d67b2b8ac3dcb0295f21aaaa90000781904faa', class: "content-navigation-block__skip-content" }, index.h("slot", { key: 'd2b48b2a5303addf5ba96446b0d62ab58df63bd9', name: "skipContent" })), index.h("div", { key: 'f761d179d17aba23f753e06b0053c51b5e211d0a', ref: el => (this.openButton = el), class: "content-navigation__open" }, !this.hasOpenButton && (index.h("button", { key: '1e1ccdc95f16b1a67993dd027cec01bc5acdb4a6' }, index.h("svg", { key: '81160138c5ba378e1636fa7cad9eca021d3adfc1', class: "default-icon", xmlns: "http://www.w3.org/2000/svg", fill: "none", viewBox: "0 0 24 24", "stroke-width": "1.5", stroke: "currentColor" }, index.h("path", { key: '839d1da86a66cf9cc5d859ac18e5fcdf07378cd8', "stroke-linecap": "round", "stroke-linejoin": "round", d: "M3.75 6.75h16.5M3.75 12h16.5m-16.5 5.25h16.5" })))), index.h("slot", { key: 'c5f1cbae43972510679119a34a020ae9f3fcfebb', name: "openButton" })), index.h("div", { key: '7a46c5d6e94682c3c766f5e3d855f7cbb5bd1a30', class: "content-navigation__content-wrapper" }, index.h("div", { key: 'b97fff1a6b0865771580cf04adc248af19ca62de', ref: el => (this.closeButton = el), class: "content-navigation__close" }, !this.hasCloseButton && (index.h("button", { key: '79b6601fe41e9b04d14b79f8a3861a5707ea12a9' }, index.h("svg", { key: 'd896c72d9aed7383968dd17da37ee1348a453805', class: "default-icon", xmlns: "http://www.w3.org/2000/svg", fill: "none", viewBox: "0 0 24 24", "stroke-width": "1.5", stroke: "currentColor" }, index.h("path", { key: '12322b22ce35445878f82c3f2d90a8c9de6ed835', "stroke-linecap": "round", "stroke-linejoin": "round", d: "M6 18 18 6M6 6l12 12" })))), index.h("slot", { key: 'd90e629a54928181707f53286bfc660113ac603a', name: "closeButton" })), index.h("div", { key: 'f2ba632063737c2ef5e0ddfb4db81e223036a2fc', class: "content-navigation__content", ref: el => (this.contentSlot = el) }, index.h("slot", { key: 'f3bf538a642cb87013b66b757fe545e32ee17c2e', name: "content" }), index.h("slot", { key: 'c4d0af20cc31af068d827b5c07bccf625cdf52e0' })))));
    }
    get hostElement() { return index.getElement(this); }
};
ContentNavigationBlock.style = ContentNavigationBlockStyle0;

exports.content_navigation_block = ContentNavigationBlock;

//# sourceMappingURL=content-navigation-block.cjs.entry.js.map