import { h, Host } from "@stencil/core";
import { WindowResizeService } from "../../services/window-resize-service";
/**
 * @slot background - This slot is usually either a video or image component.
 * @slot content - This slot will usually serve a text block.
 * @slot - Default slot, same as content slot
 */
export class ContentHeroBlock {
    constructor() {
        this.hasBackground = false;
        this.contentPosition = 'below|above';
        this.classes = undefined;
        this.classString = undefined;
        this.revealHiddenContent = false;
    }
    componentWillLoad() {
        this.hasBackground = !!this.hostElement.querySelector('[slot="background"]');
        this.hasHiddenContent = !!this.hostElement.querySelector('[slot="activatedContent"]');
        WindowResizeService.registerProp(this.contentPosition).subscribe(newValue => (this.contentPosition = newValue));
        WindowResizeService.registerProp(this.classes).subscribe(newValue => (this.classString = newValue));
    }
    contentBlockActivationHandler(event) {
        if (this.hostElement === event.detail) {
            this.hostElement.setAttribute('activated', 'true');
        }
    }
    render() {
        return (h(Host, { key: '453139dde7f5f37fe4b246b65f85c5b4252eb25b', class: this.classString }, this.hasBackground && (h("div", { key: '61af0bcd5932c1678424f8d8d39b9374c848eac5', class: "content-hero-block__background" }, h("slot", { key: '5b565590f57803ab199f847aa37b221df4795b8b', name: "background" }))), h("div", { key: 'c25011a540f9d35d3635d47fca259aeefc1b6ca6', class: "content-hero-block__content" }, h("slot", { key: '8c4e12fcb9a805445b879f8ac642e2894c1a2b31', name: "content" }), h("slot", { key: '75560112753c4b2ab546bf5caf2b32701917e6c6' }))));
    }
    static get is() { return "content-hero-block"; }
    static get originalStyleUrls() {
        return {
            "$": ["content-hero-block.scss"]
        };
    }
    static get styleUrls() {
        return {
            "$": ["content-hero-block.css"]
        };
    }
    static get properties() {
        return {
            "contentPosition": {
                "type": "string",
                "mutable": true,
                "complexType": {
                    "original": "ResponsiveString | HeroContentPlacementOptions",
                    "resolved": "string",
                    "references": {
                        "ResponsiveString": {
                            "location": "import",
                            "path": "../../typings",
                            "id": "src/typings.d.ts::ResponsiveString"
                        },
                        "HeroContentPlacementOptions": {
                            "location": "local",
                            "path": "/Users/shahrukceo/Developer/content-blocks-monorepo/packages/content-blocks-stenciljs/src/blocks/content-hero-block/content-hero-block.tsx",
                            "id": "src/blocks/content-hero-block/content-hero-block.tsx::HeroContentPlacementOptions"
                        }
                    }
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [{
                            "name": "type",
                            "text": "Select"
                        }, {
                            "name": "options",
                            "text": "below"
                        }, {
                            "name": "options",
                            "text": "above"
                        }, {
                            "name": "responsive",
                            "text": undefined
                        }],
                    "text": "Pipe separated HeroContentPlacementOptions that will determine whether the content sits below the media slot or overlapped above."
                },
                "attribute": "content-position",
                "reflect": true,
                "defaultValue": "'below|above'"
            },
            "classes": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "ResponsiveString",
                    "resolved": "string",
                    "references": {
                        "ResponsiveString": {
                            "location": "import",
                            "path": "../../typings",
                            "id": "src/typings.d.ts::ResponsiveString"
                        }
                    }
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [{
                            "name": "type",
                            "text": "String"
                        }, {
                            "name": "responsive",
                            "text": undefined
                        }],
                    "text": ""
                },
                "attribute": "classes",
                "reflect": false
            }
        };
    }
    static get states() {
        return {
            "hasBackground": {},
            "classString": {},
            "revealHiddenContent": {}
        };
    }
    static get elementRef() { return "hostElement"; }
    static get listeners() {
        return [{
                "name": "open",
                "method": "contentBlockActivationHandler",
                "target": "window",
                "capture": false,
                "passive": false
            }];
    }
}
//# sourceMappingURL=content-hero-block.js.map
