import { h, Host } from "@stencil/core";
import { WindowResizeService } from "../../services/window-resize-service";
/**
 * @slot content - This slot will usually serve a text block.
 * @slot footer - This slot is placed below the content slot
 * @slot - Default slot, same as content slot
 */
export class ContentTextBlock {
    constructor() {
        this.classes = undefined;
        this.classString = undefined;
        this.minHeight = undefined;
        this.vertical = 'top';
        this.fullHeight = 'false';
        this.textAlign = 'default';
        this.inlineStyle = {};
    }
    componentWillLoad() {
        if (!!this.hostElement.querySelector('[slot="footer"]')) {
            this.hostElement.setAttribute('footer', 'true');
        }
        WindowResizeService.registerProp(this.minHeight).subscribe(newValue => {
            this.inlineStyle = Object.assign(Object.assign({}, this.inlineStyle), { 'min-height': isNaN(newValue) ? newValue : newValue + 'px' });
        });
        WindowResizeService.registerProp(this.textAlign).subscribe(newValue => this.hostElement.setAttribute('text-align', newValue));
        WindowResizeService.registerProp(this.vertical).subscribe(newValue => this.hostElement.setAttribute('vertical', newValue));
        WindowResizeService.registerProp(this.fullHeight).subscribe(newValue => {
            this.hostElement.setAttribute('full-height', newValue);
        });
        WindowResizeService.registerProp(this.classes).subscribe(newValue => (this.classString = newValue));
    }
    render() {
        return (h(Host, { key: '116ebc47c5f89f06424fbe67e76052160fc2291d', class: this.classString, style: this.inlineStyle }, h("div", { key: '31647851675daf77d948fd1b401302d79b134051', class: "content-text-block__content" }, h("slot", { key: '33b09d8504765218eb85883ba44435247a375593', name: "content" }), h("slot", { key: '3a6b285567c0023f1e4ada56df5e2ad3c418861c' })), h("slot", { key: '689f93bc46db8c458a0fa0d827503cbc2449da9a', name: "footer" })));
    }
    static get is() { return "content-text-block"; }
    static get originalStyleUrls() {
        return {
            "$": ["content-text-block.scss"]
        };
    }
    static get styleUrls() {
        return {
            "$": ["content-text-block.css"]
        };
    }
    static get properties() {
        return {
            "classes": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "ResponsiveString",
                    "resolved": "string",
                    "references": {
                        "ResponsiveString": {
                            "location": "import",
                            "path": "../../typings",
                            "id": "src/typings.d.ts::ResponsiveString"
                        }
                    }
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [{
                            "name": "type",
                            "text": "String"
                        }, {
                            "name": "responsive",
                            "text": undefined
                        }],
                    "text": "This is a pipe separated string representing the style class."
                },
                "attribute": "classes",
                "reflect": false
            },
            "minHeight": {
                "type": "string",
                "mutable": true,
                "complexType": {
                    "original": "ResponsiveString",
                    "resolved": "string",
                    "references": {
                        "ResponsiveString": {
                            "location": "import",
                            "path": "../../typings",
                            "id": "src/typings.d.ts::ResponsiveString"
                        }
                    }
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [{
                            "name": "type",
                            "text": "String"
                        }, {
                            "name": "responsive",
                            "text": undefined
                        }],
                    "text": "This is a pipe separated string representing the minimum height. Units without values default to pixels."
                },
                "attribute": "min-height",
                "reflect": true
            },
            "vertical": {
                "type": "string",
                "mutable": true,
                "complexType": {
                    "original": "ContentVerticalAlignments",
                    "resolved": "\"bottom\" | \"center\" | \"top\"",
                    "references": {
                        "ContentVerticalAlignments": {
                            "location": "import",
                            "path": "../../typings",
                            "id": "src/typings.d.ts::ContentVerticalAlignments"
                        }
                    }
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [{
                            "name": "type",
                            "text": "Select"
                        }, {
                            "name": "options",
                            "text": "top"
                        }, {
                            "name": "options",
                            "text": "center"
                        }, {
                            "name": "options",
                            "text": "bottom"
                        }, {
                            "name": "responsive",
                            "text": undefined
                        }],
                    "text": "Pipe separated list of ContentVerticalAlignments for text box placement."
                },
                "attribute": "vertical",
                "reflect": true,
                "defaultValue": "'top'"
            },
            "fullHeight": {
                "type": "string",
                "mutable": true,
                "complexType": {
                    "original": "ResponsiveString",
                    "resolved": "string",
                    "references": {
                        "ResponsiveString": {
                            "location": "import",
                            "path": "../../typings",
                            "id": "src/typings.d.ts::ResponsiveString"
                        }
                    }
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [{
                            "name": "type",
                            "text": "Boolean"
                        }, {
                            "name": "responsive",
                            "text": undefined
                        }],
                    "text": ""
                },
                "attribute": "full-height",
                "reflect": true,
                "defaultValue": "'false'"
            },
            "textAlign": {
                "type": "string",
                "mutable": true,
                "complexType": {
                    "original": "ContentLineTextAlignOptions",
                    "resolved": "\"center\" | \"default\" | \"left\" | \"right\"",
                    "references": {
                        "ContentLineTextAlignOptions": {
                            "location": "import",
                            "path": "../../typings",
                            "id": "src/typings.d.ts::ContentLineTextAlignOptions"
                        }
                    }
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [{
                            "name": "type",
                            "text": "Select"
                        }, {
                            "name": "options",
                            "text": "default"
                        }, {
                            "name": "options",
                            "text": "left"
                        }, {
                            "name": "options",
                            "text": "center"
                        }, {
                            "name": "options",
                            "text": "right"
                        }, {
                            "name": "responsive",
                            "text": undefined
                        }],
                    "text": "Pipe separated list of ContentLineTextAlignOptions for text alignment."
                },
                "attribute": "text-align",
                "reflect": true,
                "defaultValue": "'default'"
            }
        };
    }
    static get states() {
        return {
            "classString": {},
            "inlineStyle": {}
        };
    }
    static get elementRef() { return "hostElement"; }
}
//# sourceMappingURL=content-text-block.js.map
