import { h, Host } from "@stencil/core";
import { WindowResizeService } from "../../services/window-resize-service";
/**
 * @slot content - This slot will usually serve a content button.
 * @slot trigger - This is the contents of the button that will expand / hide the accordion.
 * @slot - Default slot, same as content slot.
 */
export class ContentAccordion {
    constructor() {
        this.revealElement = undefined;
        this.open = 'false';
        this.animationStart = 'animation-start';
        this.animationEnd = 'animation-end';
        this.triggerClass = '';
        this.animationTime = 0;
        this.isOpen = false;
        this.randomID = '';
        this.classString = '';
    }
    handleClickEvent(e) {
        this.animateAccordion(!this.isOpen);
    }
    animateAccordion(open, animationTime = this.animationTime) {
        if (this.timeoutHolder) {
            clearTimeout(this.timeoutHolder);
        }
        if (open) {
            this.isOpen = true;
            this.timeoutHolder = setTimeout(() => {
                this.hostElement.classList.add(this.animationStart);
            }, 0);
        }
        else {
            this.hostElement.classList.add(this.animationEnd);
            this.timeoutHolder = setTimeout(() => {
                this.hostElement.classList.remove(this.animationStart, this.animationEnd);
                this.isOpen = false;
            }, animationTime);
        }
    }
    componentWillLoad() {
        this.revealElement = this.hostElement.querySelector('.js-content-accordion-reveal');
        if (this.hostElement.id) {
            this.randomID = `${this.hostElement.tagName}_${this.hostElement.id}`;
        }
        else {
            this.randomID = `${this.hostElement.tagName}_${Math.floor(Math.random() * document.querySelectorAll(this.hostElement.tagName).length)}`;
        }
        WindowResizeService.registerProp(this.open).subscribe(newValue => {
            this.isOpen = newValue === 'true';
            if (this.isOpen) {
                this.animateAccordion(true);
            }
        });
    }
    render() {
        return (h(Host, { key: 'e2c74da4422159599813e36f564a13e2632e1fd4', class: this.classString, expanded: this.isOpen }, h("button", { key: '9096a0b08f0d1dddb0e7993d2156ea434dcff50f', class: this.triggerClass, onClick: e => this.handleClickEvent(e), "aria-expanded": this.isOpen, "aria-controls": this.randomID }, h("slot", { key: '70246b4f77596110f4acdff1efaaae9e55011023', name: "trigger" })), h("div", { key: '3bfc780073eb30a1d90b6d48ced09e0f178f6e02', id: this.randomID, class: "js-content-accordion-reveal content-accordion__reveal" }, h("slot", { key: '0a16c47e42329443d8228483a0e23d26563aeb3c', name: "content" }), h("slot", { key: '0119746c82beb4872b597b2546d3a1ffebb51002' }))));
    }
    static get is() { return "content-accordion"; }
    static get originalStyleUrls() {
        return {
            "$": ["content-accordion.scss"]
        };
    }
    static get styleUrls() {
        return {
            "$": ["content-accordion.css"]
        };
    }
    static get properties() {
        return {
            "open": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "ResponsiveBoolean",
                    "resolved": "string",
                    "references": {
                        "ResponsiveBoolean": {
                            "location": "import",
                            "path": "../../typings",
                            "id": "src/typings.d.ts::ResponsiveBoolean"
                        }
                    }
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [{
                            "name": "type",
                            "text": "Boolean"
                        }, {
                            "name": "responsive",
                            "text": undefined
                        }],
                    "text": ""
                },
                "attribute": "open",
                "reflect": false,
                "defaultValue": "'false'"
            },
            "animationStart": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [{
                            "name": "type",
                            "text": "String"
                        }],
                    "text": "The starting animation class in case you want to control content styling."
                },
                "attribute": "animation-start",
                "reflect": false,
                "defaultValue": "'animation-start'"
            },
            "animationEnd": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [{
                            "name": "type",
                            "text": "String"
                        }],
                    "text": "The ending animation class in case you want to control content styling."
                },
                "attribute": "animation-end",
                "reflect": false,
                "defaultValue": "'animation-end'"
            },
            "triggerClass": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [{
                            "name": "type",
                            "text": "String"
                        }],
                    "text": "The class to be applied to the trigger button element."
                },
                "attribute": "trigger-class",
                "reflect": false,
                "defaultValue": "''"
            },
            "animationTime": {
                "type": "number",
                "mutable": false,
                "complexType": {
                    "original": "number",
                    "resolved": "number",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [{
                            "name": "type",
                            "text": "Number"
                        }],
                    "text": "The amount of time in milliseconds to run the closing animation, before the class is removed"
                },
                "attribute": "animation-time",
                "reflect": false,
                "defaultValue": "0"
            }
        };
    }
    static get states() {
        return {
            "revealElement": {},
            "isOpen": {},
            "randomID": {},
            "classString": {}
        };
    }
    static get elementRef() { return "hostElement"; }
}
//# sourceMappingURL=content-accordion.js.map
