import { h, Host } from "@stencil/core";
import { BehaviorSubject } from "rxjs";
export class ContentCarousel {
    constructor() {
        this.loadStatus = new BehaviorSubject(false);
        this.initMethod = 'load';
        this.swiperOptions = '{"slidesPerView": 1, "slidesPerGroup": 1, "autoplay": true, "loop": false}';
    }
    getSwiperOptions(newOptions) {
        return Object.assign({
            observer: true,
            observeParents: true,
            pagination: {
                el: this.swiperPagination,
            },
            navigation: {
                nextEl: this.swiperNextElement,
                prevEl: this.swiperPrevElement,
            },
            scrollbar: {
                el: this.swiperScrollbar,
            },
        }, newOptions);
    }
    componentDidLoad() {
        if (this.initMethod === 'load') {
            if (this.swiperOptions) {
                this.swiperOptionsFinal = this.getSwiperOptions(JSON.parse(this.swiperOptions || ''));
            }
            let options = {
                root: null,
                rootMargin: '0px',
                threshold: 0.1,
            };
            let observer = new IntersectionObserver(entries => {
                entries.forEach(async (entry) => {
                    if (entry.isIntersecting && !this.loadStatus.getValue()) {
                        await this.initSwiper();
                        this.loadStatus.next(true);
                    }
                });
            }, options);
            observer.observe(this.hostElement);
        }
    }
    setSwiperOptionsCallback(event) {
        this.loadStatus.subscribe(async () => {
            let loaded = this.loadStatus.value;
            if (loaded) {
                this.swiperOptionsFinal = this.getSwiperOptions(event.detail);
                await this.initSwiper();
            }
        });
    }
    async initSwiper() {
        let Swiper = await import('swiper');
        let SwiperModules = await import('swiper/modules');
        for (const slide of this.swiperWrapper.children) {
            slide.classList.add('swiper-slide');
        }
        // configure Swiper to use modules
        this.swiperCarousel = new Swiper.Swiper(this.swiper, Object.assign(Object.assign({}, this.swiperOptionsFinal), {
            modules: [SwiperModules.A11y, SwiperModules.Navigation, SwiperModules.Pagination],
        }));
    }
    render() {
        return (h(Host, { key: '52e7364f9399eaee4fbf6d405a1f48802b98eb22' }, h("div", { key: '964fc12db2a07cba5dd33f946bf6ae8278d7fcf0', class: "swiper", ref: el => (this.swiper = el) }, h("div", { key: '8e591ae0b46c7e0aabf0d99b37549ad8847f7bdc', class: "swiper-wrapper", ref: el => (this.swiperWrapper = el) }, h("slot", { key: '22ee7442bd47b549e5550b552c9008587b29d4b4', name: "content" }), h("slot", { key: '1e15f9d3bcfdd2a382bd642dabe180726873ac84' })), h("div", { key: '1587e736eda960cd398b3b03845ab32c22053729', class: "swiper-pagination", ref: el => (this.swiperPagination = el) }), h("button", { key: 'c3364d5b40396808cb79726d38fd822dfe34573f', class: "swiper-button-prev", ref: el => (this.swiperPrevElement = el) }), h("button", { key: 'd595d5d5805adb73f33f095d6f22e11fed6eb812', class: "swiper-button-next", ref: el => (this.swiperNextElement = el) }), h("div", { key: '0256d63beec47b84cb31c7aa4682daf24d5a1724', class: "swiper-scrollbar", ref: el => (this.swiperScrollbar = el) }))));
    }
    static get is() { return "content-carousel"; }
    static get originalStyleUrls() {
        return {
            "$": ["content-carousel.scss"]
        };
    }
    static get styleUrls() {
        return {
            "$": ["content-carousel.css"]
        };
    }
    static get properties() {
        return {
            "initMethod": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "InitMethodType",
                    "resolved": "\"event\" | \"load\"",
                    "references": {
                        "InitMethodType": {
                            "location": "local",
                            "path": "/Users/shahrukceo/Developer/content-blocks-monorepo/packages/content-blocks-stenciljs/src/components/content-carousel/content-carousel.tsx",
                            "id": "src/components/content-carousel/content-carousel.tsx::InitMethodType"
                        }
                    }
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [{
                            "name": "type",
                            "text": "Select"
                        }, {
                            "name": "options",
                            "text": "load"
                        }, {
                            "name": "options",
                            "text": "init"
                        }],
                    "text": ""
                },
                "attribute": "init-method",
                "reflect": false,
                "defaultValue": "'load'"
            },
            "swiperOptions": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [{
                            "name": "type",
                            "text": "JSON"
                        }, {
                            "name": "schema",
                            "text": "Swiper Options"
                        }],
                    "text": ""
                },
                "attribute": "swiper-options",
                "reflect": false,
                "defaultValue": "'{\"slidesPerView\": 1, \"slidesPerGroup\": 1, \"autoplay\": true, \"loop\": false}'"
            }
        };
    }
    static get elementRef() { return "hostElement"; }
    static get listeners() {
        return [{
                "name": "setSwiperOptions",
                "method": "setSwiperOptionsCallback",
                "target": undefined,
                "capture": false,
                "passive": false
            }];
    }
}
//# sourceMappingURL=content-carousel.js.map
