import { h, Host } from "@stencil/core";
import { WindowResizeService } from "../../services/window-resize-service";
/**
 * @slot content - Inner contents of the button, usually plaintext.
 * @slot - Default slot, same as content slot.
 * @slot activatedContent - If this slot is used, the content in this will be scrolled to and triggered.
 */
export class ContentButton {
    constructor() {
        this.href = undefined;
        this.target = undefined;
        this.classes = '';
        this.classString = '';
        this.clickableBlock = false;
        this.expanded = undefined;
        this.controls = undefined;
        this.linkLabel = undefined;
        this.linkClass = undefined;
        this.hasContentToActivate = undefined;
    }
    componentWillLoad() {
        this.activatedContent = this.hostElement.querySelector('[slot="activatedContent"]');
        this.hasContentToActivate = !!this.activatedContent;
        // If this is a clickable block, forcefully add a # to force an anchor link
        if (!this.href || this.href === '#') {
            this.href = 'javascript:void(0);';
        }
        WindowResizeService.registerProp(this.linkClass).subscribe(newValue => {
            this.linkClass = newValue;
        });
        WindowResizeService.registerProp(this.classes).subscribe(newValue => {
            this.classString = newValue;
        });
    }
    onHandleClickEvent(ev) {
        if (this.hasContentToActivate) {
            ev.preventDefault();
            // ev.stopPropagation();
            this.activatedContent.dispatchEvent(new CustomEvent('open', { detail: this.activatedContent }));
            setTimeout(() => {
                this.activatedContent.scrollIntoView({ behavior: 'smooth' });
            }, 0);
        }
        else if (this.href && this.href.length > 1 && this.href.substring(0, 1) === '#') {
            let element = document.getElementById(this.href.substring(1));
            ev.preventDefault();
            if (element) {
                ev.preventDefault();
                // ev.stopPropagation();
                element.dispatchEvent(new CustomEvent('open', { detail: element }));
                setTimeout(() => {
                    element.scrollIntoView({ behavior: 'smooth' });
                }, 0);
            }
        }
    }
    render() {
        return (h(Host, { key: '32645cce0c5d64972be5614a9d8c08af7212dca8', class: this.classString }, h("a", { key: 'd199235db6ba6690db54e4021938ee8b3b57fe74', "aria-label": this.linkLabel, class: this.linkClass, role: this.href === 'javascript:void(0);' ? 'button' : '', onClick: ev => this.onHandleClickEvent(ev), "aria-expanded": this.expanded, "aria-controls": this.controls, href: this.href, target: this.target }, !this.clickableBlock ? (h("span", null, h("slot", { name: "content" }), h("slot", null))) : (h("div", { class: "content-button__clickable-content" }, h("slot", { name: "content" }), h("slot", null)))), h("slot", { key: '424adf7f2642fd10291a2974a4a41e8148240654', name: "activatedContent" })));
    }
    static get is() { return "content-button"; }
    static get originalStyleUrls() {
        return {
            "$": ["content-button.scss"]
        };
    }
    static get styleUrls() {
        return {
            "$": ["content-button.css"]
        };
    }
    static get properties() {
        return {
            "href": {
                "type": "string",
                "mutable": true,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [{
                            "name": "type",
                            "text": "URL"
                        }],
                    "text": ""
                },
                "attribute": "href",
                "reflect": true
            },
            "target": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [{
                            "name": "type",
                            "text": "String"
                        }],
                    "text": ""
                },
                "attribute": "target",
                "reflect": false
            },
            "classes": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [{
                            "name": "type",
                            "text": "String"
                        }, {
                            "name": "responsive",
                            "text": undefined
                        }],
                    "text": "The classes for the button."
                },
                "attribute": "classes",
                "reflect": false,
                "defaultValue": "''"
            },
            "clickableBlock": {
                "type": "boolean",
                "mutable": false,
                "complexType": {
                    "original": "boolean",
                    "resolved": "boolean",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [{
                            "name": "type",
                            "text": "Boolean"
                        }],
                    "text": ""
                },
                "attribute": "clickable-block",
                "reflect": false,
                "defaultValue": "false"
            },
            "expanded": {
                "type": "string",
                "mutable": true,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [{
                            "name": "type",
                            "text": "Boolean"
                        }],
                    "text": ""
                },
                "attribute": "expanded",
                "reflect": true
            },
            "controls": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [{
                            "name": "type",
                            "text": "Target"
                        }],
                    "text": "\n/**\n  Aria-controls value"
                },
                "attribute": "controls",
                "reflect": false
            },
            "linkLabel": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "ResponsiveString",
                    "resolved": "string",
                    "references": {
                        "ResponsiveString": {
                            "location": "import",
                            "path": "../../typings",
                            "id": "src/typings.d.ts::ResponsiveString"
                        }
                    }
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [{
                            "name": "type",
                            "text": "String"
                        }, {
                            "name": "responsive",
                            "text": undefined
                        }],
                    "text": ""
                },
                "attribute": "link-label",
                "reflect": false
            },
            "linkClass": {
                "type": "string",
                "mutable": true,
                "complexType": {
                    "original": "ResponsiveString",
                    "resolved": "string",
                    "references": {
                        "ResponsiveString": {
                            "location": "import",
                            "path": "../../typings",
                            "id": "src/typings.d.ts::ResponsiveString"
                        }
                    }
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [{
                            "name": "type",
                            "text": "String"
                        }, {
                            "name": "responsive",
                            "text": undefined
                        }],
                    "text": ""
                },
                "attribute": "link-class",
                "reflect": true
            }
        };
    }
    static get states() {
        return {
            "classString": {},
            "hasContentToActivate": {}
        };
    }
    static get elementRef() { return "hostElement"; }
}
//# sourceMappingURL=content-button.js.map
