import { h, Host } from "@stencil/core";
import { WindowResizeService } from "../../services/window-resize-service";
import { getCurrentValue } from "../../utils/utils";
/**
 * @slot mediaContent - Content displayed over media. If you want to reveal content on hover, use a tout block
 */
export class ContentImage {
    constructor() {
        this.classes = undefined;
        this.lazyLoad = true;
        this.src = undefined;
        this.activeSrc = undefined;
        this.alt = undefined;
        this.width = '16';
        this.height = '9';
        this.aspectRatio = undefined;
        this.type = 'image';
        this.videoType = 'video/mp4';
        this.showContent = 'false|true';
        this.lazyLoadIsComplete = false;
        this.showVideoAsGif = false;
        this.activeLazyLoadClass = '';
        this.classString = '';
        this.inlineStyle = {};
        this.hasMediaContent = false;
        this.pictureImageSrc = undefined;
    }
    preloadImage(src) {
        return new Promise(r => {
            const image = new Image();
            image.onload = r;
            image.onerror = r;
            image.src = src;
        });
    }
    componentWillLoad() {
        this.hasMediaContent = !!this.hostElement.querySelector('[slot="mediaContent"]');
        WindowResizeService.registerProp(this.aspectRatio).subscribe(newValue => {
            this.inlineStyle = Object.assign(Object.assign({}, this.inlineStyle), { '--aspect-ratio': `${newValue}` });
        });
        WindowResizeService.registerProp(this.alt).subscribe(newValue => {
            this.alt = newValue;
        });
        WindowResizeService.registerProp(this.height).subscribe(newValue => {
            this.height = newValue;
        });
        WindowResizeService.registerProp(this.showContent).subscribe(newValue => {
            this.showContent = newValue;
        });
        WindowResizeService.registerProp(this.classes).subscribe(newValue => {
            this.classString = `${newValue}`;
        });
    }
    componentDidLoad() {
        let observer = new IntersectionObserver(entries => {
            entries.forEach(async (entry) => {
                if (entry.isIntersecting) {
                    this.lazyLoadIsComplete = true;
                    this.pictureImageSrc = getCurrentValue(this.src);
                    this.showVideoAsGif = this.pictureImageSrc.indexOf('.mp4') >= 0 || getCurrentValue(this.type) === 'video';
                }
            });
        }, {
            root: null,
            rootMargin: `${Math.max(100, window.innerHeight * 1)}px`,
            // rootMargin: `0px`,
            threshold: 0,
        });
        observer.observe(this.hostElement);
    }
    watchLazyLoadIsComplete(newValue) {
        if (newValue === true) {
            this.activeLazyLoadClass = 'content-image-loaded';
        }
        else {
            if (!this.lazyLoad) {
                this.activeLazyLoadClass = 'content-image-no-lazyload';
            }
            else {
                this.activeLazyLoadClass = 'content-image-loading';
            }
        }
    }
    render() {
        return (h(Host, { key: '7a29a891d8d81398e91b2cbdd6c7e6fb7c2c6b3b', class: `${this.activeLazyLoadClass} ${this.classString}`, style: this.inlineStyle, content: this.showContent }, this.hasMediaContent && (h("div", { key: '44b05149260cf7e3449b0e03843dfc220989dd4d', class: "content-image__media-content" }, h("slot", { key: '225ded4cd89e04b7c7c71969cc391567b449a682', name: "mediaContent" }))), this.showVideoAsGif && (h("video", { key: '5b379113637e6a9b077a5572e3b97b48de244b4d', autoplay: true, muted: true, loop: true, preload: "none", playsinline: true, width: this.width, height: this.height }, h("source", { key: '4348f929ee94dd2e0194fbbeba31f1a69569d87a', src: this.lazyLoadIsComplete ? this.pictureImageSrc : null, type: this.videoType }))), this.pictureImageSrc && !this.showVideoAsGif && h("img", { key: '57a1310b8185e6a8f0bd592d63094c535699364a', width: this.width, height: this.height, src: this.pictureImageSrc, alt: this.alt })));
    }
    static get is() { return "content-image"; }
    static get originalStyleUrls() {
        return {
            "$": ["content-image.scss"]
        };
    }
    static get styleUrls() {
        return {
            "$": ["content-image.css"]
        };
    }
    static get properties() {
        return {
            "classes": {
                "type": "string",
                "mutable": true,
                "complexType": {
                    "original": "ResponsiveString",
                    "resolved": "string",
                    "references": {
                        "ResponsiveString": {
                            "location": "import",
                            "path": "../../typings",
                            "id": "src/typings.d.ts::ResponsiveString"
                        }
                    }
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [{
                            "name": "type",
                            "text": "String"
                        }, {
                            "name": "responsive",
                            "text": undefined
                        }],
                    "text": ""
                },
                "attribute": "classes",
                "reflect": true
            },
            "lazyLoad": {
                "type": "boolean",
                "mutable": false,
                "complexType": {
                    "original": "boolean",
                    "resolved": "boolean",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [{
                            "name": "type",
                            "text": "Boolean"
                        }],
                    "text": ""
                },
                "attribute": "lazy-load",
                "reflect": false,
                "defaultValue": "true"
            },
            "src": {
                "type": "string",
                "mutable": true,
                "complexType": {
                    "original": "ResponsiveString",
                    "resolved": "string",
                    "references": {
                        "ResponsiveString": {
                            "location": "import",
                            "path": "../../typings",
                            "id": "src/typings.d.ts::ResponsiveString"
                        }
                    }
                },
                "required": true,
                "optional": false,
                "docs": {
                    "tags": [{
                            "name": "type",
                            "text": "Media"
                        }, {
                            "name": "responsive",
                            "text": undefined
                        }],
                    "text": ""
                },
                "attribute": "src",
                "reflect": false
            },
            "alt": {
                "type": "string",
                "mutable": true,
                "complexType": {
                    "original": "ResponsiveString",
                    "resolved": "string",
                    "references": {
                        "ResponsiveString": {
                            "location": "import",
                            "path": "../../typings",
                            "id": "src/typings.d.ts::ResponsiveString"
                        }
                    }
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [{
                            "name": "type",
                            "text": "String"
                        }, {
                            "name": "responsive",
                            "text": undefined
                        }],
                    "text": ""
                },
                "attribute": "alt",
                "reflect": true
            },
            "width": {
                "type": "string",
                "mutable": true,
                "complexType": {
                    "original": "ResponsiveNumber",
                    "resolved": "string",
                    "references": {
                        "ResponsiveNumber": {
                            "location": "import",
                            "path": "../../typings",
                            "id": "src/typings.d.ts::ResponsiveNumber"
                        }
                    }
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [{
                            "name": "type",
                            "text": "Number"
                        }, {
                            "name": "responsive",
                            "text": undefined
                        }],
                    "text": ""
                },
                "attribute": "width",
                "reflect": false,
                "defaultValue": "'16'"
            },
            "height": {
                "type": "string",
                "mutable": true,
                "complexType": {
                    "original": "ResponsiveNumber",
                    "resolved": "string",
                    "references": {
                        "ResponsiveNumber": {
                            "location": "import",
                            "path": "../../typings",
                            "id": "src/typings.d.ts::ResponsiveNumber"
                        }
                    }
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [{
                            "name": "type",
                            "text": "Number"
                        }, {
                            "name": "responsive",
                            "text": undefined
                        }],
                    "text": ""
                },
                "attribute": "height",
                "reflect": false,
                "defaultValue": "'9'"
            },
            "aspectRatio": {
                "type": "string",
                "mutable": true,
                "complexType": {
                    "original": "ResponsiveString",
                    "resolved": "string",
                    "references": {
                        "ResponsiveString": {
                            "location": "import",
                            "path": "../../typings",
                            "id": "src/typings.d.ts::ResponsiveString"
                        }
                    }
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [{
                            "name": "type",
                            "text": "String"
                        }, {
                            "name": "responsive",
                            "text": undefined
                        }],
                    "text": ""
                },
                "attribute": "aspect-ratio",
                "reflect": false
            },
            "type": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "'image' | 'video'",
                    "resolved": "\"image\" | \"video\"",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [{
                            "name": "type",
                            "text": "Select"
                        }, {
                            "name": "options",
                            "text": "image"
                        }, {
                            "name": "options",
                            "text": "video"
                        }],
                    "text": ""
                },
                "attribute": "type",
                "reflect": false,
                "defaultValue": "'image'"
            },
            "videoType": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [{
                            "name": "type",
                            "text": "Select"
                        }, {
                            "name": "options",
                            "text": "video/mp4"
                        }],
                    "text": ""
                },
                "attribute": "video-type",
                "reflect": false,
                "defaultValue": "'video/mp4'"
            },
            "showContent": {
                "type": "string",
                "mutable": true,
                "complexType": {
                    "original": "ResponsiveBoolean",
                    "resolved": "string",
                    "references": {
                        "ResponsiveBoolean": {
                            "location": "import",
                            "path": "../../typings",
                            "id": "src/typings.d.ts::ResponsiveBoolean"
                        }
                    }
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [{
                            "name": "type",
                            "text": "Boolean"
                        }, {
                            "name": "responsive",
                            "text": undefined
                        }],
                    "text": "If true, the content in the mediaContent slot will be visible."
                },
                "attribute": "show-content",
                "reflect": true,
                "defaultValue": "'false|true'"
            }
        };
    }
    static get states() {
        return {
            "activeSrc": {},
            "lazyLoadIsComplete": {},
            "showVideoAsGif": {},
            "activeLazyLoadClass": {},
            "classString": {},
            "inlineStyle": {},
            "hasMediaContent": {},
            "pictureImageSrc": {}
        };
    }
    static get elementRef() { return "hostElement"; }
    static get watchers() {
        return [{
                "propName": "lazyLoadIsComplete",
                "methodName": "watchLazyLoadIsComplete"
            }];
    }
}
//# sourceMappingURL=content-image.js.map
