import { h, Host } from "@stencil/core";
import { WindowResizeService } from "../../services/window-resize-service";
/**
 * @slot content - Inner contents of the line, usually plaintext.
 * @slot - Default slot, same as content slot.
 */
export class ContentLine {
    constructor() {
        this.tag = 'div';
        this.textAlign = 'default';
        this.classes = undefined;
        this.classString = undefined;
        this.maxWidth = '100%';
        this.inlineStyle = {};
    }
    componentWillLoad() {
        WindowResizeService.registerProp(this.textAlign).subscribe(newValue => {
            this.inlineStyle = Object.assign(Object.assign({}, this.inlineStyle), { '--text-align': newValue });
        });
        WindowResizeService.registerProp(this.maxWidth).subscribe((newValue) => {
            this.inlineStyle = Object.assign(Object.assign({}, this.inlineStyle), { '--max-width': isNaN(newValue) ? newValue : newValue + '%' });
        });
        WindowResizeService.registerProp(this.classes).subscribe(newValue => {
            this.classString = newValue;
        });
    }
    render() {
        // The capital case in this JSX variable is important
        const ElementTagParsed = `${this.tag}`;
        return (h(Host, { key: 'f810a0a780d8bc063d340114a2ad0817de12e235', class: this.classString, style: this.inlineStyle }, h(ElementTagParsed, { key: '1b003bf1daa171d768597de9d47443e286fd20fa', class: "content-line__inner" }, h("slot", { key: 'aead059a6b1421d0e84a56c72c84a3d24889f126', name: "content" }), h("slot", { key: 'eda8aaa9eebdd2ac42ef0768c4b4ebe1fe5d6927' }))));
    }
    static get is() { return "content-line"; }
    static get originalStyleUrls() {
        return {
            "$": ["content-line.scss"]
        };
    }
    static get styleUrls() {
        return {
            "$": ["content-line.css"]
        };
    }
    static get properties() {
        return {
            "tag": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "ContentLineTagOptions",
                    "resolved": "\"article\" | \"div\" | \"h1\" | \"h2\" | \"h3\" | \"h4\" | \"h5\" | \"h6\" | \"p\" | \"section\" | \"span\"",
                    "references": {
                        "ContentLineTagOptions": {
                            "location": "local",
                            "path": "/Users/shahrukceo/Developer/content-blocks-monorepo/packages/content-blocks-stenciljs/src/elements/content-line/content-line.tsx",
                            "id": "src/elements/content-line/content-line.tsx::ContentLineTagOptions"
                        }
                    }
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [{
                            "name": "type",
                            "text": "Select"
                        }, {
                            "name": "options",
                            "text": "div"
                        }, {
                            "name": "options",
                            "text": "p"
                        }, {
                            "name": "options",
                            "text": "h1"
                        }, {
                            "name": "options",
                            "text": "h2"
                        }, {
                            "name": "options",
                            "text": "h3"
                        }, {
                            "name": "options",
                            "text": "h4"
                        }, {
                            "name": "options",
                            "text": "h5"
                        }, {
                            "name": "options",
                            "text": "h6"
                        }, {
                            "name": "options",
                            "text": "span"
                        }, {
                            "name": "options",
                            "text": "section"
                        }, {
                            "name": "options",
                            "text": "article"
                        }],
                    "text": "The element tag to use."
                },
                "attribute": "tag",
                "reflect": false,
                "defaultValue": "'div'"
            },
            "textAlign": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "ContentLineTextAlignOptions",
                    "resolved": "\"center\" | \"default\" | \"left\" | \"right\"",
                    "references": {
                        "ContentLineTextAlignOptions": {
                            "location": "local",
                            "path": "/Users/shahrukceo/Developer/content-blocks-monorepo/packages/content-blocks-stenciljs/src/elements/content-line/content-line.tsx",
                            "id": "src/elements/content-line/content-line.tsx::ContentLineTextAlignOptions"
                        }
                    }
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [{
                            "name": "type",
                            "text": "Select"
                        }, {
                            "name": "options",
                            "text": "default"
                        }, {
                            "name": "options",
                            "text": "left"
                        }, {
                            "name": "options",
                            "text": "center"
                        }, {
                            "name": "options",
                            "text": "right"
                        }, {
                            "name": "responsive",
                            "text": undefined
                        }],
                    "text": "Pipe separated list of ContentLineTextAlignOptions for text alignment."
                },
                "attribute": "text-align",
                "reflect": false,
                "defaultValue": "'default'"
            },
            "classes": {
                "type": "any",
                "mutable": false,
                "complexType": {
                    "original": "any",
                    "resolved": "any",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [{
                            "name": "type",
                            "text": "String"
                        }, {
                            "name": "responsive",
                            "text": undefined
                        }],
                    "text": "The font class for the line."
                },
                "attribute": "classes",
                "reflect": false
            },
            "maxWidth": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "ResponsiveNumber",
                    "resolved": "string",
                    "references": {
                        "ResponsiveNumber": {
                            "location": "import",
                            "path": "../../typings",
                            "id": "src/typings.d.ts::ResponsiveNumber"
                        }
                    }
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [{
                            "name": "type",
                            "text": "String"
                        }, {
                            "name": "responsive",
                            "text": undefined
                        }],
                    "text": "Pipe separated list of max width numbers. You can also use units such as \"px\" and \"vw\". If no unit is provided, defaults to percentage."
                },
                "attribute": "max-width",
                "reflect": false,
                "defaultValue": "'100%'"
            }
        };
    }
    static get states() {
        return {
            "classString": {},
            "inlineStyle": {}
        };
    }
}
//# sourceMappingURL=content-line.js.map
