import { h, Host } from "@stencil/core";
import { WindowResizeService } from "../../services/window-resize-service";
import { getCurrentValue } from "../../utils/utils";
import videojs from "video.js";
import * as videojsyoutube from "videojs-youtube";
videojsyoutube; // needed to ensure videojs youtube loads
/**
 * @slot poster - Poster shown before video is played.
 * @slot playButton - Play button contents.
 * @slot no-js-message - Contents of no JS message.
 * @slot content - Adjacent to no-js message for video. Usually not needed.
 * @slot - Default slot, same as content slot.
 */
export class ContentVideo {
    constructor() {
        this.method = 'youtube';
        this.autoplay = false;
        this.autoplayMute = true;
        this.src = '';
        this.width = 16;
        this.height = 9;
        this.type = 'video/mp4';
        this.playButtonAriaLabel = 'Play';
        this.classes = undefined;
        this.classString = undefined;
        this.controls = true;
        this.preload = 'metadata';
        this.fitContainer = 'false';
        this.isLoaded = false;
        this.hasPoster = false;
        this.isPlayed = false;
    }
    computedVideoConfig() {
        let videoConfig = {};
        videoConfig = {
            fill: getCurrentValue(this.fitContainer) === 'true',
            fluid: getCurrentValue(this.fitContainer) === 'false',
            autoplay: this.autoplay === true ? (this.autoplayMute ? 'muted' : 'any') : false,
            techOrder: ['youtube'],
            src: this.src,
            sources: [
                {
                    type: 'video/youtube',
                    src: this.src,
                },
            ],
            youtube: {
                showinfo: 0,
                rel: 0,
                modestbranding: 1,
            },
        };
        console.log(videoConfig);
        return videoConfig;
    }
    initVideo() {
        this.isLoaded = true;
    }
    componentWillLoad() {
        WindowResizeService.registerProp(this.classes).subscribe(newValue => (this.classString = newValue));
    }
    componentDidLoad() {
        let self = this;
        let options = {
            root: null,
            rootMargin: '0px',
            threshold: 0.1,
        };
        this.player = videojs(this.videoElement);
        // let observer = new IntersectionObserver(entries => {
        //   entries.forEach(async entry => {
        //     if (entry.isIntersecting && !this.player) {
        //       // if (!videojs) {
        //       //   videojs = await import('video.js');
        //       //   await import('videojs-youtube/dist/Youtube.js');
        //       // }
        //       // this.hasPoster = !!this.hostElement.querySelector('[slot="poster"]');
        //       // if (this.videoElement && !this.player) {
        //         // this.player = videojs.default(this.videoElement, this.computedVideoConfig(), function onPlayerReady() {
        //         //   self.initVideo();
        //         //   if (self.hasPoster || self.autoplay) {
        //         //     self.isPlayed = true;
        //         //     setTimeout(() => this.play(), 1);
        //         //   }
        //         // });
        //       // }
        //     }
        //   });
        // }, options);
        // observer.observe(this.hostElement);
    }
    onClickHandler(event) {
        event.preventDefault();
        if (!this.isLoaded) {
            return false;
        }
        this.initVideo();
        this.isPlayed = true;
        this.player.play();
        if (this.method === 'html5') {
            this.videoElement.play();
        }
    }
    render() {
        let showPlayButton = !this.autoplay && ((this.hasPoster && !this.isPlayed) || !this.isPlayed);
        return (h(Host, { key: '22cffbd35601f37a8bbc0bd5eda02cda1688c9cc', class: this.classString }, this.hasPoster && !this.isLoaded && (h("div", { key: 'a49d09ea914a8fc90c3dc10bab99e74347fad000', class: "content-video__poster" }, h("slot", { key: 'b1e57d86ac11a2b03950bb9ffb40f73253ac838b', name: "poster" }))), showPlayButton && (h("button", { key: '14dbd5508a52b66ab112d2dbfd1d5de3b96310eb', class: "content-video__play", "aria-label": this.playButtonAriaLabel, onClick: ev => this.onClickHandler(ev) }, h("slot", { key: '6ae7662d21f79e8c05c5d17652cbb7e37533cb5d', name: "playButton" }))), h("div", { key: '3b942a282fd0df1e22569fd02ff248065f67374e', class: "content-video__content" }, (!this.hasPoster || this.isLoaded) && (h("video", { key: 'd93067f22489d1afcfefcd7413e81c41e37d0c37', ref: element => (this.videoElement = element), class: "video-js vjs-content-block-theme", controls: this.controls, preload: this.preload, width: this.width, height: this.height, crossorigin: "anonymous", "data-setup": JSON.stringify(this.computedVideoConfig()) }, h("slot", { key: '8c784e20d7938cfb890d2877d208bbf514735ad4', name: "no-js-message" }), h("slot", { key: '351930777416a2bbc6a89a37e2e69f241a3dbf9c', name: "content" }), h("slot", { key: 'f06a2361d6397fca3d15a9a5533e94667a73bf15' }))))));
    }
    static get is() { return "content-video"; }
    static get originalStyleUrls() {
        return {
            "$": ["content-video.scss"]
        };
    }
    static get styleUrls() {
        return {
            "$": ["content-video.css"]
        };
    }
    static get properties() {
        return {
            "method": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "SupportedVideoMethods",
                    "resolved": "\"html5\" | \"youtube\"",
                    "references": {
                        "SupportedVideoMethods": {
                            "location": "local",
                            "path": "/Users/shahrukceo/Developer/content-blocks-monorepo/packages/content-blocks-stenciljs/src/elements/content-video/content-video.tsx",
                            "id": "src/elements/content-video/content-video.tsx::SupportedVideoMethods"
                        }
                    }
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [{
                            "name": "type",
                            "text": "Select"
                        }, {
                            "name": "options",
                            "text": "youtube"
                        }, {
                            "name": "options",
                            "text": "html5"
                        }],
                    "text": ""
                },
                "attribute": "method",
                "reflect": false,
                "defaultValue": "'youtube'"
            },
            "autoplay": {
                "type": "boolean",
                "mutable": false,
                "complexType": {
                    "original": "boolean",
                    "resolved": "boolean",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [{
                            "name": "type",
                            "text": "Boolean"
                        }],
                    "text": "If you're using this inside a content-overlay for example, you may want the video to play immediately on launch."
                },
                "attribute": "autoplay",
                "reflect": false,
                "defaultValue": "false"
            },
            "autoplayMute": {
                "type": "boolean",
                "mutable": false,
                "complexType": {
                    "original": "boolean",
                    "resolved": "boolean",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [{
                            "name": "type",
                            "text": "Boolean"
                        }],
                    "text": "Sets whether or not an autoplaying video should be muted. \nBy default autoplay videos will be muted."
                },
                "attribute": "autoplay-mute",
                "reflect": false,
                "defaultValue": "true"
            },
            "src": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [{
                            "name": "type",
                            "text": "String"
                        }],
                    "text": "Supports YouTube and self hosted URLs."
                },
                "attribute": "src",
                "reflect": false,
                "defaultValue": "''"
            },
            "width": {
                "type": "number",
                "mutable": false,
                "complexType": {
                    "original": "number",
                    "resolved": "number",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [{
                            "name": "type",
                            "text": "Number"
                        }],
                    "text": ""
                },
                "attribute": "width",
                "reflect": false,
                "defaultValue": "16"
            },
            "height": {
                "type": "number",
                "mutable": false,
                "complexType": {
                    "original": "number",
                    "resolved": "number",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [{
                            "name": "type",
                            "text": "Number"
                        }],
                    "text": ""
                },
                "attribute": "height",
                "reflect": false,
                "defaultValue": "9"
            },
            "type": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [{
                            "name": "type",
                            "text": "String"
                        }],
                    "text": ""
                },
                "attribute": "type",
                "reflect": false,
                "defaultValue": "'video/mp4'"
            },
            "playButtonAriaLabel": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [{
                            "name": "type",
                            "text": "String"
                        }],
                    "text": ""
                },
                "attribute": "play-button-aria-label",
                "reflect": false,
                "defaultValue": "'Play'"
            },
            "classes": {
                "type": "string",
                "mutable": true,
                "complexType": {
                    "original": "ResponsiveString",
                    "resolved": "string",
                    "references": {
                        "ResponsiveString": {
                            "location": "import",
                            "path": "../../typings",
                            "id": "src/typings.d.ts::ResponsiveString"
                        }
                    }
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [{
                            "name": "type",
                            "text": "String"
                        }, {
                            "name": "responsive",
                            "text": undefined
                        }],
                    "text": ""
                },
                "attribute": "classes",
                "reflect": true
            },
            "controls": {
                "type": "boolean",
                "mutable": false,
                "complexType": {
                    "original": "boolean",
                    "resolved": "boolean",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [{
                            "name": "type",
                            "text": "Boolean"
                        }],
                    "text": ""
                },
                "attribute": "controls",
                "reflect": false,
                "defaultValue": "true"
            },
            "preload": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [{
                            "name": "type",
                            "text": "String"
                        }],
                    "text": ""
                },
                "attribute": "preload",
                "reflect": false,
                "defaultValue": "'metadata'"
            },
            "fitContainer": {
                "type": "string",
                "mutable": true,
                "complexType": {
                    "original": "ResponsiveString",
                    "resolved": "string",
                    "references": {
                        "ResponsiveString": {
                            "location": "import",
                            "path": "../../typings",
                            "id": "src/typings.d.ts::ResponsiveString"
                        }
                    }
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [{
                            "name": "responsive",
                            "text": undefined
                        }, {
                            "name": "type",
                            "text": "String"
                        }],
                    "text": "Fit the container this video is placed in. When the video is used in an overlay or another place where the video will takeover the existing height, it is recommended to enable this option."
                },
                "attribute": "fit-container",
                "reflect": true,
                "defaultValue": "'false'"
            }
        };
    }
    static get states() {
        return {
            "classString": {},
            "isLoaded": {},
            "hasPoster": {},
            "isPlayed": {}
        };
    }
    static get elementRef() { return "hostElement"; }
}
//# sourceMappingURL=content-video.js.map
