import { h, Host } from "@stencil/core";
import { WindowResizeService } from "../../services/window-resize-service";
/**
 * @slot media - This slot can either be a video or image component.
 * @slot content - This slot will usually serve a content text block.
 */
export class ContentGridFormatter {
    constructor() {
        this.classes = undefined;
        this.classString = undefined;
        this.type = 'flex';
        this.flexGrow = 'false';
        this.flexShrink = 'false';
        this.vertical = 'space-between';
        this.horizontal = 'space-between';
        this.columns = '1';
        this.columnGap = '0';
        this.rowGap = '0';
        this.activeClassString = undefined;
        this.inlineStyle = {};
    }
    componentWillLoad() {
        return Promise.allSettled([
            WindowResizeService.registerProp(this.type).subscribe(newValue => (this.type = newValue)),
            WindowResizeService.registerProp(this.vertical).subscribe(newValue => {
                this.vertical = newValue;
                this.inlineStyle = Object.assign(Object.assign({}, this.inlineStyle), { '--align-items': this.vertical });
            }),
            WindowResizeService.registerProp(this.horizontal).subscribe(newValue => {
                this.horizontal = newValue;
                this.inlineStyle = Object.assign(Object.assign({}, this.inlineStyle), { '--justify-content': this.horizontal });
            }),
            WindowResizeService.registerProp(this.flexShrink).subscribe(newValue => (this.flexShrink = newValue)),
            WindowResizeService.registerProp(this.flexGrow).subscribe(newValue => (this.flexGrow = newValue)),
            WindowResizeService.registerProp(this.columns).subscribe(newValue => {
                this.columns = newValue;
                this.inlineStyle = Object.assign(Object.assign({}, this.inlineStyle), { '--columns': newValue });
            }),
            WindowResizeService.registerProp(this.columnGap).subscribe(newValue => {
                this.columnGap = newValue;
                this.inlineStyle = Object.assign(Object.assign({}, this.inlineStyle), { '--column-gap': newValue });
            }),
            WindowResizeService.registerProp(this.rowGap).subscribe(newValue => {
                this.rowGap = newValue;
                this.inlineStyle = Object.assign(Object.assign({}, this.inlineStyle), { '--row-gap': newValue });
            }),
            WindowResizeService.registerProp(this.classes).subscribe(newValue => (this.classString = newValue)),
        ]);
    }
    render() {
        return (h(Host, { key: '0f4c01fb1b73a7b9fa3994a198d56e43fbadd7cc', class: this.classString, style: this.inlineStyle }, h("slot", { key: 'cf16d48ab340f5612a66e189d936dba2e69ccee7' })));
    }
    static get is() { return "content-grid-formatter"; }
    static get originalStyleUrls() {
        return {
            "$": ["content-grid-formatter.scss"]
        };
    }
    static get styleUrls() {
        return {
            "$": ["content-grid-formatter.css"]
        };
    }
    static get properties() {
        return {
            "classes": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "ResponsiveString",
                    "resolved": "string",
                    "references": {
                        "ResponsiveString": {
                            "location": "import",
                            "path": "../../typings",
                            "id": "src/typings.d.ts::ResponsiveString"
                        }
                    }
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [{
                            "name": "type",
                            "text": "String"
                        }, {
                            "name": "responsive",
                            "text": undefined
                        }],
                    "text": ""
                },
                "attribute": "classes",
                "reflect": false
            },
            "type": {
                "type": "string",
                "mutable": true,
                "complexType": {
                    "original": "GRID_TYPES | ResponsiveString",
                    "resolved": "string",
                    "references": {
                        "GRID_TYPES": {
                            "location": "import",
                            "path": "../../typings",
                            "id": "src/typings.d.ts::GRID_TYPES"
                        },
                        "ResponsiveString": {
                            "location": "import",
                            "path": "../../typings",
                            "id": "src/typings.d.ts::ResponsiveString"
                        }
                    }
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [{
                            "name": "type",
                            "text": "Select"
                        }, {
                            "name": "options",
                            "text": "none"
                        }, {
                            "name": "options",
                            "text": "flex"
                        }, {
                            "name": "options",
                            "text": "grid"
                        }, {
                            "name": "responsive",
                            "text": undefined
                        }],
                    "text": "Flex or grid"
                },
                "attribute": "type",
                "reflect": true,
                "defaultValue": "'flex'"
            },
            "flexGrow": {
                "type": "string",
                "mutable": true,
                "complexType": {
                    "original": "ResponsiveBoolean",
                    "resolved": "string",
                    "references": {
                        "ResponsiveBoolean": {
                            "location": "import",
                            "path": "../../typings",
                            "id": "src/typings.d.ts::ResponsiveBoolean"
                        }
                    }
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [{
                            "name": "type",
                            "text": "Boolean"
                        }, {
                            "name": "responsive",
                            "text": undefined
                        }],
                    "text": "Setting this true to will grow all children. Alternatively, add the attribute [flex-grow=\"true\"] to a specific grid child and that child will grow."
                },
                "attribute": "flex-grow",
                "reflect": true,
                "defaultValue": "'false'"
            },
            "flexShrink": {
                "type": "string",
                "mutable": true,
                "complexType": {
                    "original": "ResponsiveBoolean",
                    "resolved": "string",
                    "references": {
                        "ResponsiveBoolean": {
                            "location": "import",
                            "path": "../../typings",
                            "id": "src/typings.d.ts::ResponsiveBoolean"
                        }
                    }
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [{
                            "name": "type",
                            "text": "Boolean"
                        }, {
                            "name": "responsive",
                            "text": undefined
                        }],
                    "text": "Setting this true to will shrink all children. Alternatively, add the attribute [flex-shrink=\"true\"] to a specific grid child and that child will shrink."
                },
                "attribute": "flex-shrink",
                "reflect": true,
                "defaultValue": "'false'"
            },
            "vertical": {
                "type": "string",
                "mutable": true,
                "complexType": {
                    "original": "FLEX_JUSTIFY_OPTIONS",
                    "resolved": "\"center\" | \"flex-end\" | \"flex-start\" | \"space-between\" | \"space-evenly\"",
                    "references": {
                        "FLEX_JUSTIFY_OPTIONS": {
                            "location": "import",
                            "path": "../../typings",
                            "id": "src/typings.d.ts::FLEX_JUSTIFY_OPTIONS"
                        }
                    }
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [{
                            "name": "type",
                            "text": "Select"
                        }, {
                            "name": "options",
                            "text": "flex-start"
                        }, {
                            "name": "options",
                            "text": "space-between"
                        }, {
                            "name": "options",
                            "text": "center"
                        }, {
                            "name": "options",
                            "text": "space-evenly"
                        }, {
                            "name": "responsive",
                            "text": undefined
                        }],
                    "text": "Vertical axis alignment"
                },
                "attribute": "vertical",
                "reflect": true,
                "defaultValue": "'space-between'"
            },
            "horizontal": {
                "type": "string",
                "mutable": true,
                "complexType": {
                    "original": "FLEX_JUSTIFY_OPTIONS",
                    "resolved": "\"center\" | \"flex-end\" | \"flex-start\" | \"space-between\" | \"space-evenly\"",
                    "references": {
                        "FLEX_JUSTIFY_OPTIONS": {
                            "location": "import",
                            "path": "../../typings",
                            "id": "src/typings.d.ts::FLEX_JUSTIFY_OPTIONS"
                        }
                    }
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [{
                            "name": "type",
                            "text": "Select"
                        }, {
                            "name": "options",
                            "text": "flex-start"
                        }, {
                            "name": "options",
                            "text": "space-between"
                        }, {
                            "name": "options",
                            "text": "center"
                        }, {
                            "name": "options",
                            "text": "space-evenly"
                        }, {
                            "name": "responsive",
                            "text": undefined
                        }],
                    "text": "Horizontal axis alignment"
                },
                "attribute": "horizontal",
                "reflect": true,
                "defaultValue": "'space-between'"
            },
            "columns": {
                "type": "string",
                "mutable": true,
                "complexType": {
                    "original": "ResponsiveNumber",
                    "resolved": "string",
                    "references": {
                        "ResponsiveNumber": {
                            "location": "import",
                            "path": "../../typings",
                            "id": "src/typings.d.ts::ResponsiveNumber"
                        }
                    }
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [{
                            "name": "type",
                            "text": "Number"
                        }, {
                            "name": "responsive",
                            "text": undefined
                        }],
                    "text": ""
                },
                "attribute": "columns",
                "reflect": true,
                "defaultValue": "'1'"
            },
            "columnGap": {
                "type": "string",
                "mutable": true,
                "complexType": {
                    "original": "ResponsiveNumber",
                    "resolved": "string",
                    "references": {
                        "ResponsiveNumber": {
                            "location": "import",
                            "path": "../../typings",
                            "id": "src/typings.d.ts::ResponsiveNumber"
                        }
                    }
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [{
                            "name": "type",
                            "text": "Number"
                        }, {
                            "name": "responsive",
                            "text": undefined
                        }],
                    "text": ""
                },
                "attribute": "column-gap",
                "reflect": true,
                "defaultValue": "'0'"
            },
            "rowGap": {
                "type": "string",
                "mutable": true,
                "complexType": {
                    "original": "ResponsiveNumber",
                    "resolved": "string",
                    "references": {
                        "ResponsiveNumber": {
                            "location": "import",
                            "path": "../../typings",
                            "id": "src/typings.d.ts::ResponsiveNumber"
                        }
                    }
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [{
                            "name": "type",
                            "text": "Number"
                        }, {
                            "name": "responsive",
                            "text": undefined
                        }],
                    "text": ""
                },
                "attribute": "row-gap",
                "reflect": true,
                "defaultValue": "'0'"
            }
        };
    }
    static get states() {
        return {
            "classString": {},
            "activeClassString": {},
            "inlineStyle": {}
        };
    }
    static get elementRef() { return "hostElement"; }
}
//# sourceMappingURL=content-grid-formatter.js.map
