import { h, Host } from "@stencil/core";
import { WindowResizeService } from "../../services/window-resize-service";
/**
 * @slot content - Inner contents of spacing wrapper, can be anything.
 * @slot - Default slot, same as content slot.
 */
export class ContentSpacingWrapper {
    constructor() {
        this.classes = undefined;
        this.classString = undefined;
        this.maxWidth = '100';
        this.fullHeight = 'false';
        this.textAlign = 'default';
        this.horizontal = 'default';
        this.vertical = '';
        this.topOffset = '0';
        this.bottomOffset = '0';
        this.leftOffset = '0';
        this.rightOffset = '0';
        this.inlineStyle = {};
    }
    componentWillLoad() {
        WindowResizeService.registerProp(this.fullHeight).subscribe(newValue => (this.fullHeight = newValue));
        WindowResizeService.registerProp(this.textAlign).subscribe(newValue => (this.textAlign = newValue));
        WindowResizeService.registerProp(this.horizontal).subscribe(newValue => (this.horizontal = newValue));
        WindowResizeService.registerProp(this.vertical).subscribe(newValue => (this.vertical = newValue));
        WindowResizeService.registerProp(this.maxWidth).subscribe(newValue => {
            this.inlineStyle = Object.assign(Object.assign({}, this.inlineStyle), { 'max-width': isNaN(newValue) ? newValue : newValue + '%' });
        });
        WindowResizeService.registerProp(this.topOffset).subscribe(newValue => {
            this.inlineStyle = Object.assign(Object.assign({}, this.inlineStyle), { 'padding-top': isNaN(newValue) ? newValue : newValue + '%' });
        });
        WindowResizeService.registerProp(this.bottomOffset).subscribe(newValue => {
            this.inlineStyle = Object.assign(Object.assign({}, this.inlineStyle), { 'padding-bottom': isNaN(newValue) ? newValue : newValue + '%' });
        });
        WindowResizeService.registerProp(this.leftOffset).subscribe(newValue => {
            this.inlineStyle = Object.assign(Object.assign({}, this.inlineStyle), { 'padding-left': isNaN(newValue) ? newValue : newValue + '%' });
        });
        WindowResizeService.registerProp(this.rightOffset).subscribe(newValue => {
            this.inlineStyle = Object.assign(Object.assign({}, this.inlineStyle), { 'padding-right': isNaN(newValue) ? newValue : newValue + '%' });
        });
        WindowResizeService.registerProp(this.classes).subscribe(newValue => (this.classString = newValue));
    }
    render() {
        return (h(Host, { key: 'd77bbf3aceceac17d3a2b139b631d6b44f618176', class: this.classString, style: this.inlineStyle }, h("div", { key: '54ab935fa44a783060e67780a68e5ae2d5ad216a', class: "content-spacer__content" }, h("slot", { key: 'dee136bc8ce06c4445a1df2536735cc756ee5cb2', name: "content" }), h("slot", { key: '244e9656209c313c4ae989c5ad49995dd66c9638' }))));
    }
    static get is() { return "content-spacer"; }
    static get originalStyleUrls() {
        return {
            "$": ["content-spacer.scss"]
        };
    }
    static get styleUrls() {
        return {
            "$": ["content-spacer.css"]
        };
    }
    static get properties() {
        return {
            "classes": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "ResponsiveString",
                    "resolved": "string",
                    "references": {
                        "ResponsiveString": {
                            "location": "import",
                            "path": "../../typings",
                            "id": "src/typings.d.ts::ResponsiveString"
                        }
                    }
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [{
                            "name": "type",
                            "text": "String"
                        }, {
                            "name": "responsive",
                            "text": undefined
                        }],
                    "text": ""
                },
                "attribute": "classes",
                "reflect": false
            },
            "maxWidth": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "ResponsiveNumber",
                    "resolved": "string",
                    "references": {
                        "ResponsiveNumber": {
                            "location": "import",
                            "path": "../../typings",
                            "id": "src/typings.d.ts::ResponsiveNumber"
                        }
                    }
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [{
                            "name": "type",
                            "text": "String"
                        }, {
                            "name": "responsive",
                            "text": undefined
                        }],
                    "text": "Pipe separated list of max width numbers. You can also use units such as \"px\" and \"vw\"."
                },
                "attribute": "max-width",
                "reflect": false,
                "defaultValue": "'100'"
            },
            "fullHeight": {
                "type": "string",
                "mutable": true,
                "complexType": {
                    "original": "ResponsiveBoolean",
                    "resolved": "string",
                    "references": {
                        "ResponsiveBoolean": {
                            "location": "import",
                            "path": "../../typings",
                            "id": "src/typings.d.ts::ResponsiveBoolean"
                        }
                    }
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [{
                            "name": "type",
                            "text": "Boolean"
                        }, {
                            "name": "responsive",
                            "text": undefined
                        }],
                    "text": "Pipe separated boolean values for full height. Typically, this is used if you have a hero block and you want a footer that aligns with the text layer. This is a niche case, but a valid one."
                },
                "attribute": "full-height",
                "reflect": true,
                "defaultValue": "'false'"
            },
            "textAlign": {
                "type": "string",
                "mutable": true,
                "complexType": {
                    "original": "ContentLineTextAlignOptions",
                    "resolved": "\"center\" | \"default\" | \"left\" | \"right\"",
                    "references": {
                        "ContentLineTextAlignOptions": {
                            "location": "import",
                            "path": "../../typings",
                            "id": "src/typings.d.ts::ContentLineTextAlignOptions"
                        }
                    }
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [{
                            "name": "type",
                            "text": "Select"
                        }, {
                            "name": "options",
                            "text": "default"
                        }, {
                            "name": "options",
                            "text": "left"
                        }, {
                            "name": "options",
                            "text": "center"
                        }, {
                            "name": "options",
                            "text": "right"
                        }, {
                            "name": "responsive",
                            "text": undefined
                        }],
                    "text": "Pipe separated list of ContentLineTextAlignOptions for text alignment."
                },
                "attribute": "text-align",
                "reflect": true,
                "defaultValue": "'default'"
            },
            "horizontal": {
                "type": "string",
                "mutable": true,
                "complexType": {
                    "original": "ContentLineTextAlignOptions",
                    "resolved": "\"center\" | \"default\" | \"left\" | \"right\"",
                    "references": {
                        "ContentLineTextAlignOptions": {
                            "location": "import",
                            "path": "../../typings",
                            "id": "src/typings.d.ts::ContentLineTextAlignOptions"
                        }
                    }
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [{
                            "name": "type",
                            "text": "Select"
                        }, {
                            "name": "options",
                            "text": "default"
                        }, {
                            "name": "options",
                            "text": "left"
                        }, {
                            "name": "options",
                            "text": "center"
                        }, {
                            "name": "options",
                            "text": "right"
                        }, {
                            "name": "responsive",
                            "text": undefined
                        }],
                    "text": "Pipe separated list of ContentLineTextAlignOptions for where to place text box when a max width is specified."
                },
                "attribute": "horizontal",
                "reflect": true,
                "defaultValue": "'default'"
            },
            "vertical": {
                "type": "string",
                "mutable": true,
                "complexType": {
                    "original": "ResponsiveString | ContentVerticalAlignments",
                    "resolved": "string",
                    "references": {
                        "ResponsiveString": {
                            "location": "import",
                            "path": "../../typings",
                            "id": "src/typings.d.ts::ResponsiveString"
                        },
                        "ContentVerticalAlignments": {
                            "location": "import",
                            "path": "../../typings",
                            "id": "src/typings.d.ts::ContentVerticalAlignments"
                        }
                    }
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [{
                            "name": "type",
                            "text": "Select"
                        }, {
                            "name": "options",
                            "text": "top"
                        }, {
                            "name": "options",
                            "text": "center"
                        }, {
                            "name": "options",
                            "text": "bottom"
                        }, {
                            "name": "options",
                            "text": "default"
                        }, {
                            "name": "responsive",
                            "text": undefined
                        }],
                    "text": "Pipe separated list of ContentVerticalAlignments for text box placement."
                },
                "attribute": "vertical",
                "reflect": true,
                "defaultValue": "''"
            },
            "topOffset": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "ResponsiveNumber",
                    "resolved": "string",
                    "references": {
                        "ResponsiveNumber": {
                            "location": "import",
                            "path": "../../typings",
                            "id": "src/typings.d.ts::ResponsiveNumber"
                        }
                    }
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [{
                            "name": "type",
                            "text": "Number"
                        }, {
                            "name": "string",
                            "text": undefined
                        }, {
                            "name": "responsive",
                            "text": undefined
                        }],
                    "text": "Pipe separated numbers between 0 and 100. Units without values are assumed to be %."
                },
                "attribute": "top-offset",
                "reflect": false,
                "defaultValue": "'0'"
            },
            "bottomOffset": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "ResponsiveNumber",
                    "resolved": "string",
                    "references": {
                        "ResponsiveNumber": {
                            "location": "import",
                            "path": "../../typings",
                            "id": "src/typings.d.ts::ResponsiveNumber"
                        }
                    }
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [{
                            "name": "type",
                            "text": "Number"
                        }, {
                            "name": "string",
                            "text": undefined
                        }, {
                            "name": "responsive",
                            "text": undefined
                        }],
                    "text": "Pipe separated numbers between 0 and 100. Units without values are assumed to be %."
                },
                "attribute": "bottom-offset",
                "reflect": false,
                "defaultValue": "'0'"
            },
            "leftOffset": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "ResponsiveNumber",
                    "resolved": "string",
                    "references": {
                        "ResponsiveNumber": {
                            "location": "import",
                            "path": "../../typings",
                            "id": "src/typings.d.ts::ResponsiveNumber"
                        }
                    }
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [{
                            "name": "type",
                            "text": "Number"
                        }, {
                            "name": "string",
                            "text": undefined
                        }, {
                            "name": "responsive",
                            "text": undefined
                        }],
                    "text": "Pipe separated numbers between 0 and 100. Units without values are assumed to be %."
                },
                "attribute": "left-offset",
                "reflect": false,
                "defaultValue": "'0'"
            },
            "rightOffset": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "ResponsiveNumber",
                    "resolved": "string",
                    "references": {
                        "ResponsiveNumber": {
                            "location": "import",
                            "path": "../../typings",
                            "id": "src/typings.d.ts::ResponsiveNumber"
                        }
                    }
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [{
                            "name": "type",
                            "text": "Number"
                        }, {
                            "name": "string",
                            "text": undefined
                        }, {
                            "name": "responsive",
                            "text": undefined
                        }],
                    "text": "Pipe separated numbers between 0 and 100. Units without values are assumed to be %."
                },
                "attribute": "right-offset",
                "reflect": false,
                "defaultValue": "'0'"
            }
        };
    }
    static get states() {
        return {
            "classString": {},
            "inlineStyle": {}
        };
    }
}
//# sourceMappingURL=content-spacer.js.map
