import { p as proxyCustomElement, H, d as createEvent, h, c as Host } from './p-e28db040.js';
import { W as WindowResizeService } from './p-26483b46.js';
import { c as createFocusTrap } from './p-a2fedf8a.js';

/**
 * @namespace aria
 */
var aria$1 = aria$1 || {};
aria$1.Utils = aria$1.Utils || {};
// Polyfill src https://developer.mozilla.org/en-US/docs/Web/API/Element/matches
aria$1.Utils.matches = function (element, selector) {
    return element.matches(selector);
};
aria$1.Utils.remove = function (item) {
    if (item.remove && typeof item.remove === 'function') {
        return item.remove();
    }
    if (item.parentNode &&
        item.parentNode.removeChild &&
        typeof item.parentNode.removeChild === 'function') {
        return item.parentNode.removeChild(item);
    }
    return false;
};
aria$1.Utils.isFocusable = function (element) {
    if (element.tabIndex > 0 || (element.tabIndex === 0 && element.getAttribute('tabIndex') !== null)) {
        return true;
    }
    if (element.disabled) {
        return false;
    }
    switch (element.nodeName) {
        case 'A':
            return !!element.href && element.rel != 'ignore';
        case 'INPUT':
            return element.type != 'hidden' && element.type != 'file';
        case 'BUTTON':
        case 'SELECT':
        case 'TEXTAREA':
            return true;
        default:
            return false;
    }
};
aria$1.Utils.getAncestorBySelector = function (element, selector) {
    if (!aria$1.Utils.matches(element, selector + ' ' + element.tagName)) {
        // Element is not inside an element that matches selector
        return null;
    }
    // Move up the DOM tree until a parent matching the selector is found
    var currentNode = element;
    var ancestor = null;
    while (ancestor === null) {
        if (aria$1.Utils.matches(currentNode.parentNode, selector)) {
            ancestor = currentNode.parentNode;
        }
        else {
            currentNode = currentNode.parentNode;
        }
    }
    return ancestor;
};
aria$1.Utils.hasClass = function (element, className) {
    return (new RegExp('(\\s|^)' + className + '(\\s|$)')).test(element.className);
};
aria$1.Utils.addClass = function (element, className) {
    if (!aria$1.Utils.hasClass(element, className)) {
        element.className += ' ' + className;
    }
};
aria$1.Utils.removeClass = function (element, className) {
    var classRegex = new RegExp('(\\s|^)' + className + '(\\s|$)');
    element.className = element.className.replace(classRegex, ' ').trim();
};
aria$1.Utils.bindMethods = function (object) {
    var methodNames = Array.prototype.slice.call(arguments, 1);
    methodNames.forEach(function (method) {
        object[method] = object[method].bind(object);
    });
};
window.aria = aria$1;

/*
*   This content is licensed according to the W3C Software License at
*   https://www.w3.org/Consortium/Legal/2015/copyright-software-and-document
*/
/**
 * @namespace aria
 */
var aria = aria || {};
aria.Utils = aria.Utils || {};
// Polyfill src https://developer.mozilla.org/en-US/docs/Web/API/Element/matches
aria.Utils.matches = function (element, selector) {
    return element.matches(selector);
};
aria.Utils.remove = function (item) {
    if (item.remove && typeof item.remove === 'function') {
        return item.remove();
    }
    if (item.parentNode &&
        item.parentNode.removeChild &&
        typeof item.parentNode.removeChild === 'function') {
        return item.parentNode.removeChild(item);
    }
    return false;
};
aria.Utils.isFocusable = function (element) {
    if (element.tabIndex > 0 || (element.tabIndex === 0 && element.getAttribute('tabIndex') !== null)) {
        return true;
    }
    if (element.disabled) {
        return false;
    }
    switch (element.nodeName) {
        case 'A':
            return !!element.href && element.rel != 'ignore';
        case 'INPUT':
            return element.type != 'hidden' && element.type != 'file';
        case 'BUTTON':
        case 'SELECT':
        case 'TEXTAREA':
            return true;
        default:
            return false;
    }
};
aria.Utils.getAncestorBySelector = function (element, selector) {
    if (!aria.Utils.matches(element, selector + ' ' + element.tagName)) {
        // Element is not inside an element that matches selector
        return null;
    }
    // Move up the DOM tree until a parent matching the selector is found
    var currentNode = element;
    var ancestor = null;
    while (ancestor === null) {
        if (aria.Utils.matches(currentNode.parentNode, selector)) {
            ancestor = currentNode.parentNode;
        }
        else {
            currentNode = currentNode.parentNode;
        }
    }
    return ancestor;
};
aria.Utils.hasClass = function (element, className) {
    return (new RegExp('(\\s|^)' + className + '(\\s|$)')).test(element.className);
};
aria.Utils.addClass = function (element, className) {
    if (!aria.Utils.hasClass(element, className)) {
        element.className += ' ' + className;
    }
};
aria.Utils.removeClass = function (element, className) {
    var classRegex = new RegExp('(\\s|^)' + className + '(\\s|$)');
    element.className = element.className.replace(classRegex, ' ').trim();
};
aria.Utils.bindMethods = function (object) {
    var methodNames = Array.prototype.slice.call(arguments, 1);
    methodNames.forEach(function (method) {
        object[method] = object[method].bind(object);
    });
};
/*
* When util functions move focus around, set this true so the focus listener
* can ignore the events.
*/
aria.Utils.IgnoreUtilFocusChanges = false;
aria.Utils.dialogOpenClass = 'has-dialog';
/**
 * @desc Set focus on descendant nodes until the first focusable element is
 *       found.
 * @param element
 *          DOM node for which to find the first focusable descendant.
 * @returns
 *  true if a focusable element is found and focus is set.
 */
aria.Utils.focusFirstDescendant = function (element) {
    for (var i = 0; i < element.childNodes.length; i++) {
        var child = element.childNodes[i];
        if (aria.Utils.attemptFocus(child) ||
            aria.Utils.focusFirstDescendant(child)) {
            return true;
        }
    }
    return false;
}; // end focusFirstDescendant
/**
 * @desc Find the last descendant node that is focusable.
 * @param element
 *          DOM node for which to find the last focusable descendant.
 * @returns
 *  true if a focusable element is found and focus is set.
 */
aria.Utils.focusLastDescendant = function (element) {
    for (var i = element.childNodes.length - 1; i >= 0; i--) {
        var child = element.childNodes[i];
        if (aria.Utils.attemptFocus(child) ||
            aria.Utils.focusLastDescendant(child)) {
            return true;
        }
    }
    return false;
}; // end focusLastDescendant
/**
 * @desc Set Attempt to set focus on the current node.
 * @param element
 *          The node to attempt to focus on.
 * @returns
 *  true if element is focused.
 */
aria.Utils.attemptFocus = function (element) {
    if (!aria.Utils.isFocusable(element)) {
        return false;
    }
    aria.Utils.IgnoreUtilFocusChanges = true;
    try {
        element.focus();
    }
    catch (e) {
    }
    aria.Utils.IgnoreUtilFocusChanges = false;
    return (document.activeElement === element);
}; // end attemptFocus
/* Modals can open modals. Keep track of them with this array. */
aria.OpenDialogList = aria.OpenDialogList || new Array(0);
/**
 * @returns the last opened dialog (the current dialog)
 */
aria.getCurrentDialog = function () {
    if (aria.OpenDialogList && aria.OpenDialogList.length) {
        return aria.OpenDialogList[aria.OpenDialogList.length - 1];
    }
};
aria.closeCurrentDialog = function () {
    var currentDialog = aria.getCurrentDialog();
    if (currentDialog) {
        currentDialog.close();
        return true;
    }
    return false;
};
aria.handleEscape = function (event) {
    if (event.key === 'Escape' && aria.closeCurrentDialog()) {
        event.stopPropagation();
    }
};
document.addEventListener('keyup', aria.handleEscape);
/**
 * @constructor
 * @desc Dialog object providing modal focus management.
 *
 * Assumptions: The element serving as the dialog container is present in the
 * DOM and hidden. The dialog container has role='dialog'.
 *
 * @param dialogNode
 *          The element serving as the dialog container.
 * @param focusAfterClosed
 *          Either the DOM node or the ID of the DOM node to focus when the
 *          dialog closes.
 * @param focusFirst
 *          Optional parameter containing either the DOM node or the ID of the
 *          DOM node to focus when the dialog opens. If not specified, the
 *          first focusable element in the dialog will receive focus.
 */
aria.Dialog = function (dialogNode, focusAfterClosed, focusFirst, isFullScreen) {
    this.dialogNode = dialogNode;
    if (this.dialogNode === null) {
        throw new Error('No element found');
    }
    var validRoles = ['dialog', 'alertdialog'];
    var isDialog = (this.dialogNode.getAttribute('role') || '')
        .trim()
        .split(/\s+/g)
        .some(function (token) {
        return validRoles.some(function (role) {
            return token === role;
        });
    });
    if (!isDialog) {
        throw new Error('Dialog() requires a DOM element with ARIA role of dialog or alertdialog.');
    }
    // Wrap in an individual backdrop element if one doesn't exist
    // Native <dialog> elements use the ::backdrop pseudo-element, which
    // works similarly.
    var backdropClass = 'content-overlay-background';
    if (this.dialogNode.parentNode.classList.contains(backdropClass)) {
        this.backdropNode = this.dialogNode.parentNode;
    }
    else {
        this.backdropNode = document.createElement('div');
        this.backdropNode.className = backdropClass;
        this.dialogNode.parentNode.insertBefore(this.backdropNode, this.dialogNode);
        this.backdropNode.appendChild(this.dialogNode);
    }
    this.backdropNode.classList.add('active');
    setTimeout(() => {
        this.backdropNode.classList.add('transition');
    }, 1);
    // Disable scroll on the body element
    this.isFullscreen = isFullScreen;
    if (isFullScreen) {
        document.body.classList.add(aria.Utils.dialogOpenClass);
    }
    if (typeof focusAfterClosed === 'string') {
        this.focusAfterClosed = document.getElementById(focusAfterClosed);
    }
    else if (typeof focusAfterClosed === 'object') {
        this.focusAfterClosed = focusAfterClosed;
    }
    else {
        throw new Error('the focusAfterClosed parameter is required for the aria.Dialog constructor.');
    }
    if (typeof focusFirst === 'string') {
        this.focusFirst = document.getElementById(focusFirst);
    }
    else if (typeof focusFirst === 'object') {
        this.focusFirst = focusFirst;
    }
    else {
        this.focusFirst = null;
    }
    // Bracket the dialog node with two invisible, focusable nodes.
    // While this dialog is open, we use these to make sure that focus never
    // leaves the document even if dialogNode is the first or last node.
    var preDiv = document.createElement('div');
    this.preNode = this.dialogNode.parentNode.insertBefore(preDiv, this.dialogNode);
    this.preNode.tabIndex = 0;
    var postDiv = document.createElement('div');
    this.postNode = this.dialogNode.parentNode.insertBefore(postDiv, this.dialogNode.nextSibling);
    this.postNode.tabIndex = 0;
    // If this modal is opening on top of one that is already open,
    // get rid of the document focus listener of the open dialog.
    if (aria.OpenDialogList.length > 0) {
        aria.getCurrentDialog().removeListeners();
    }
    if (isFullScreen) {
        this.addListeners();
    }
    aria.OpenDialogList.push(this);
    this.dialogNode.className = 'content-overlay-dialog'; // make visible
    if (this.focusFirst) {
        this.focusFirst.focus();
    }
    else {
        aria.Utils.focusFirstDescendant(this.dialogNode);
    }
    this.lastFocus = document.activeElement;
}; // end Dialog constructor
/**
 * @desc
 *  Hides the current top dialog,
 *  removes listeners of the top dialog,
 *  restore listeners of a parent dialog if one was open under the one that just closed,
 *  and sets focus on the element specified for focusAfterClosed.
 */
aria.Dialog.prototype.close = function () {
    aria.OpenDialogList.pop();
    this.removeListeners();
    aria.Utils.remove(this.preNode);
    aria.Utils.remove(this.postNode);
    this.dialogNode.className = 'hidden';
    this.backdropNode.classList.remove('active');
    this.backdropNode.classList.remove('transition');
    this.focusAfterClosed.focus();
    // If a dialog was open underneath this one, restore its listeners.
    if (aria.OpenDialogList.length > 0 && aria.OpenDialogList.filter(dialog => dialog.isFullscreen === true).length) {
        aria.getCurrentDialog().addListeners();
    }
    else {
        document.body.classList.remove(aria.Utils.dialogOpenClass);
    }
    this.dialogNode.dispatchEvent(new CustomEvent("ariaDialog.closed", {
        bubbles: true,
        "detail": {}
    }));
}; // end close
/**
 * @desc
 *  Hides the current dialog and replaces it with another.
 *
 * @param newDialogId
 *  ID of the dialog that will replace the currently open top dialog.
 * @param newFocusAfterClosed
 *  Optional ID or DOM node specifying where to place focus when the new dialog closes.
 *  If not specified, focus will be placed on the element specified by the dialog being replaced.
 * @param newFocusFirst
 *  Optional ID or DOM node specifying where to place focus in the new dialog when it opens.
 *  If not specified, the first focusable element will receive focus.
 */
aria.Dialog.prototype.replace = function (newDialogId, newFocusAfterClosed, newFocusFirst) {
    // var closedDialog = aria.getCurrentDialog();
    aria.OpenDialogList.pop();
    this.removeListeners();
    aria.Utils.remove(this.preNode);
    aria.Utils.remove(this.postNode);
    this.dialogNode.className = 'hidden';
    this.backdropNode.classList.remove('active');
    this.backdropNode.classList.remove('transition');
    var focusAfterClosed = newFocusAfterClosed || this.focusAfterClosed;
    new aria.Dialog(newDialogId, focusAfterClosed, newFocusFirst);
    // var dialog = new aria.Dialog(newDialogId, focusAfterClosed, newFocusFirst);
}; // end replace
aria.Dialog.prototype.addListeners = function () {
    document.addEventListener('focus', this.trapFocus, true);
}; // end addListeners
aria.Dialog.prototype.removeListeners = function () {
    document.removeEventListener('focus', this.trapFocus, true);
}; // end removeListeners
aria.Dialog.prototype.trapFocus = function (event) {
    if (aria.Utils.IgnoreUtilFocusChanges) {
        return;
    }
    var currentDialog = aria.getCurrentDialog();
    if (currentDialog.dialogNode.contains(event.target)) {
        currentDialog.lastFocus = event.target;
    }
    else {
        aria.Utils.focusFirstDescendant(currentDialog.dialogNode);
        if (currentDialog.lastFocus == document.activeElement) {
            aria.Utils.focusLastDescendant(currentDialog.dialogNode);
        }
        currentDialog.lastFocus = document.activeElement;
    }
}; // end trapFocus
aria.closeDialog = function (closeButton) {
    var topDialog = aria.getCurrentDialog();
    if (topDialog.dialogNode.contains(closeButton)) {
        topDialog.close();
    }
}; // end closeDialog
aria.replaceDialog = function (newDialogId, newFocusAfterClosed, newFocusFirst) {
    var topDialog = aria.getCurrentDialog();
    if (topDialog.dialogNode.contains(document.activeElement)) {
        topDialog.replace(newDialogId, newFocusAfterClosed, newFocusFirst);
    }
}; // end replaceDialog
window.aria = aria;

const contentOverlayCss = "*{box-sizing:border-box}.content-overlay-open--fixed{overflow:hidden}.content-block-overlay-container{position:relative;display:block;width:100%;height:100%}content-overlay{display:none;position:relative;flex:0}content-overlay .content-overlay__content>*[slot=content]{height:100%;overflow:auto}content-overlay.content-overlay-active{display:flex}content-overlay[vertical=left]{align-items:flex-start}content-overlay[vertical=center]{align-items:center}content-overlay[vertical=right]{align-items:flex-end}content-overlay[vertical=top]{justify-content:flex-start;top:0}content-overlay[vertical=center]{justify-content:center}content-overlay[vertical=bottom]{justify-content:flex-end;bottom:0}content-overlay .content-overlay-background{background:rgba(255, 255, 255, 0.6);bottom:0;display:none;top:0;z-index:100000;right:0;left:0;width:100%}content-overlay .content-overlay-background.active{display:block}content-overlay .content-overlay__header{display:flex;align-items:center}content-overlay .content-overlay__close{order:2;margin-left:auto;cursor:pointer;z-index:1;appearance:none;background:none;border:none;min-width:40px}content-overlay[position=fixed] .content-overlay-background{position:fixed}content-overlay[position=absolute]{position:absolute;box-sizing:border-box;z-index:100000;height:100%;width:100%;left:0}content-overlay[position=absolute] .content-overlay-background{position:static}content-overlay[position=absolute] .content-overlay__content,content-overlay[position=fixed] .content-overlay__content{flex:1;overflow:auto}content-overlay [role=dialog]{display:flex;flex-direction:column;height:100%;justify-content:flex-start}content-overlay [role=dialog].hidden{display:none}content-overlay[full-screen=true]{height:100%}content-overlay[full-screen=true] .content-overlay-background{height:100%}body.has-dialog{overflow:hidden}";
const ContentOverlayStyle0 = contentOverlayCss;

const ContentOverlay$1 = /*@__PURE__*/ proxyCustomElement(class ContentOverlay extends H {
    constructor() {
        super();
        this.__registerHost();
        this.contentOverlayActivated = createEvent(this, "contentOverlayActivated", 7);
        this.target = undefined;
        this.open = 'false';
        this.fullScreen = 'true';
        this.classes = undefined;
        this.horizontal = 'center';
        this.vertical = 'center';
        this.closeTitle = 'Close Overlay';
        this.position = 'fixed';
        this.isActive = false;
        this.activeClass = '';
        this.classString = '';
        this.contentSlot = undefined;
        this.isFullScreen = false;
        this.hasCloseButton = false;
        this.originalMarkup = undefined;
    }
    onContentBlockToggle(event) {
        if (this.hostElement.getAttribute('id') === event.detail.getAttribute('id')) {
            event.preventDefault();
            if (!this.isActive) {
                this.openOverlay();
            }
            else {
                this.closeOverlay();
            }
        }
    }
    onContentBlockActivated(event) {
        if (this.hostElement.getAttribute('id') === event.detail.getAttribute('id')) {
            event.preventDefault();
            this.openOverlay();
        }
    }
    onContentBlockDeactivated(event) {
        if (this.hostElement.getAttribute('id') === event.detail.getAttribute('id')) {
            event.preventDefault();
            this.closeOverlay();
        }
    }
    async openOverlay() {
        this.isActive = true;
        document.body.classList.add(`content-overlay-open`, `content-overlay-open--${this.position}`);
        document.body.classList.remove('content-overlay-close');
        if (!this.originalMarkup) {
            this.originalMarkup = this.contentSlot.innerHTML;
        }
        if (this.target) {
            let targetElement = this.target === '#' ? this.hostElement.parentElement : document.querySelector(this.target);
            if (targetElement) {
                targetElement.classList.add('content-block-overlay-container');
                targetElement.appendChild(this.hostElement);
            }
            // } else if (this.isFullScreen) {
            // if (document.querySelectorAll(`#${this.hostElement.getAttribute('id')}`).length === 1) {
            //   document.body.appendChild(this.hostElement);
            // }
        }
        if (this.originalMarkup) {
            this.contentSlot.innerHTML = this.originalMarkup;
            if (this.contentSlot.children[0] === this.contentSlot.querySelector('script[slot="content"][type="text/template"]')) {
                this.contentSlot.innerHTML = this.contentSlot.children[0].innerHTML;
            }
            this.setComputedActiveClass();
            this.focusTrap = createFocusTrap(this.hostElement, {
                fallbackFocus: this.hostElement,
                allowOutsideClick: true,
                clickOutsideDeactivates: true,
                onDeactivate: () => {
                    this.isActive = false;
                    document.body.classList.add('content-overlay-close');
                    document.body.classList.remove(`content-overlay-open`, `content-overlay-open--${this.position}`);
                    this.activeClass = '';
                    this.setComputedActiveClass();
                    this.contentSlot.innerHTML = '';
                },
            });
        }
        this.contentOverlayActivated.emit(this.hostElement);
    }
    async closeOverlay() {
        if (this.focusTrap) {
            try {
                this.focusTrap.deactivate();
            }
            catch (e) {
                console.log(e);
            }
        }
    }
    setComputedActiveClass() {
        this.activeClass = `content-overlay ${this.isActive ? 'content-overlay-active' : 'content-overlay-inactive'}  content-overlay--${this.position}`;
    }
    componentWillLoad() {
        WindowResizeService.registerProp(this.classes).subscribe(newValue => (this.classString = `${newValue}`));
        WindowResizeService.registerProp(this.position).subscribe(newValue => (this.position = newValue));
        WindowResizeService.registerProp(this.horizontal).subscribe(newValue => this.hostElement.setAttribute('horizontal', newValue));
        WindowResizeService.registerProp(this.vertical).subscribe(newValue => this.hostElement.setAttribute('vertical', newValue));
        WindowResizeService.registerProp(this.fullScreen).subscribe(newValue => {
            this.fullScreen = newValue;
            this.isFullScreen = newValue === 'true';
        });
        WindowResizeService.registerProp(this.open).subscribe(newValue => {
            this.hostElement.setAttribute('open', newValue);
        });
        this.hasCloseButton = !!this.hostElement.querySelector('[slot="closeButton"]');
    }
    closeOverlayButtonClicked(event) {
        event.preventDefault();
        this.closeOverlay();
    }
    render() {
        return (h(Host, { key: '70525d441fef22dfd54c93047776b293411de805', class: `${this.activeClass} ${this.classString}`, position: this.position }, h("div", { key: 'f69a2d0a9f62b9a8c6a73106b419023b70528152', class: "content-block-overlay-container" }, h("div", { key: '3eda44dcc57d4704258751e4287baf1223e4b393', class: `content-overlay-background ${this.isActive ? 'active' : ''}` }, h("div", { key: '5ceaa528e484ee2de56093dae89f9a6138a06f50', ref: element => (this.dialogElement = element), role: "dialog", "aria-modal": "true", class: this.isActive ? 'active' : 'hidden', "aria-expanded": this.isActive }, h("div", { key: '7af296f91887e14978090474e1e60e0807bc2684', class: "content-overlay__header" }, h("button", { key: 'e312a9a53f4d8400d8f99b3b85a252a4713d9c24', onClick: e => {
                this.closeOverlayButtonClicked(e);
            }, class: "content-overlay__close", title: this.closeTitle }, !this.hasCloseButton && (h("svg", { key: 'ab3a9eae967ae1aa65a62c4f97bf346ff5b7ae56', xmlns: "http://www.w3.org/2000/svg", fill: "none", viewBox: "0 0 24 24", "stroke-width": "1.5", stroke: "currentColor" }, h("path", { key: '66e8c160d73fa53abb9d9d1e1b10421003fd13f9', "stroke-linecap": "round", "stroke-linejoin": "round", d: "M6 18 18 6M6 6l12 12" }))), h("slot", { key: '680eda3171fd38dd5b00963eef82c1221b94d663', name: "closeButton" })), h("slot", { key: 'd8c34c5bb0e24a35b8351c3ea364c09509211c2c', name: "header" })), h("div", { key: '10415922744fe91d30afc2d7d51f379275658e0f', class: "content-overlay__content", ref: (el) => (this.contentSlot = el) }, h("slot", { key: '6564b04487863aaaeb121ab90e1361aca5fa2adc', name: "content" }), h("slot", { key: 'dac056b616dc5468eca8603c8ad7c48489224411' })))))));
    }
    componentDidRender() {
        if (this.open === 'true') {
            this.openOverlay();
        }
        if (this.isActive && this.focusTrap && !this.focusTrap.active) {
            try {
                this.focusTrap.activate();
            }
            catch (e) {
                console.log(e);
            }
        }
    }
    get hostElement() { return this; }
    static get style() { return ContentOverlayStyle0; }
}, [4, "content-overlay", {
        "target": [1],
        "open": [1537],
        "fullScreen": [1537, "full-screen"],
        "classes": [1],
        "horizontal": [1537],
        "vertical": [1537],
        "closeTitle": [1, "close-title"],
        "position": [1537],
        "isActive": [32],
        "activeClass": [32],
        "classString": [32],
        "contentSlot": [32],
        "isFullScreen": [32],
        "hasCloseButton": [32],
        "originalMarkup": [32],
        "openOverlay": [64],
        "closeOverlay": [64]
    }, [[0, "toggle", "onContentBlockToggle"], [0, "open", "onContentBlockActivated"], [0, "close", "onContentBlockDeactivated"]]]);
function defineCustomElement$1() {
    if (typeof customElements === "undefined") {
        return;
    }
    const components = ["content-overlay"];
    components.forEach(tagName => { switch (tagName) {
        case "content-overlay":
            if (!customElements.get(tagName)) {
                customElements.define(tagName, ContentOverlay$1);
            }
            break;
    } });
}
defineCustomElement$1();

const ContentOverlay = ContentOverlay$1;
const defineCustomElement = defineCustomElement$1;

export { ContentOverlay, defineCustomElement };

//# sourceMappingURL=content-overlay.js.map