import { EventEmitter, HTMLStencilElement } from '../../stencil-public-runtime';
import { ContentLineTextAlignOptions, ContentVerticalAlignments, ResponsiveBoolean, ResponsiveString } from '../../typings';
import './libraries/aria-utilities';
import './libraries/aria-dialog';
/**
 * @slot closeButton - Contents of the overlay close button.
 * @slot header - This is adjacent to the close button, usually a header or title for the overlay.
 * @slot content - Contents that sit inside the overlay.
 */
export declare class ContentOverlay {
    hostElement: HTMLStencilElement;
    /**
     * @type String
     * @responsive
     */
    target: string;
    /**
     * @type Boolean
     * @responsive
     */
    open: ResponsiveBoolean;
    /**
     * @type Boolean
     * @responsive
     */
    fullScreen: ResponsiveBoolean;
    /**
     * @type String
     * @responsive
     */
    classes: ResponsiveString;
    /**
     * A pipe delimited setting for horizontal alignment. This only applies when fullscreen mode is activated.
     * @type Select
     * @options default
     * @options left
     * @options center
     * @options right
     * @responsive
     */
    horizontal: ContentLineTextAlignOptions;
    /**
     * A pipe delimited setting for vertical alignment. This only applies when fullscreen mode is activated.
     * @type Select
     * @options top
     * @options center
     * @options bottom
     */
    vertical: ContentVerticalAlignments;
    /**
     * Title attribute value for the close button.
     * @type String
     */
    closeTitle: string;
    /**
     * Position of the overlay.
     * @type Select
     * @options absolute
     * @options fixed
     * @options static
     * @options sticky
     */
    position: ResponsiveString;
    isActive: boolean;
    activeClass: string;
    classString: string;
    contentOverlayActivated: EventEmitter<HTMLElement>;
    contentSlot: HTMLElement;
    dialogElement: HTMLDivElement;
    contentElement: HTMLDivElement;
    isFullScreen: boolean;
    hasCloseButton: boolean;
    originalMarkup: string;
    ariaDialog: any;
    focusTrap: any;
    onContentBlockToggle(event: CustomEvent): void;
    onContentBlockActivated(event: CustomEvent): void;
    onContentBlockDeactivated(event: CustomEvent): void;
    openOverlay(): Promise<void>;
    closeOverlay(): Promise<void>;
    setComputedActiveClass(): void;
    componentWillLoad(): void;
    closeOverlayButtonClicked(event: any): void;
    render(): any;
    componentDidRender(): void;
}
//# sourceMappingURL=content-overlay.d.ts.map