import { HTMLStencilElement } from '../../stencil-public-runtime';
import { ResponsiveString } from '../../typings';
export type SupportedVideoMethods = 'html5' | 'youtube';
/**
 * @slot poster - Poster shown before video is played.
 * @slot playButton - Play button contents.
 * @slot no-js-message - Contents of no JS message.
 * @slot content - Adjacent to no-js message for video. Usually not needed.
 * @slot - Default slot, same as content slot.
 */
export declare class ContentVideo {
    player: any;
    videoElement: HTMLVideoElement;
    hostElement: HTMLStencilElement;
    /**
     * @type Select
     * @options youtube
     * @options html5
     */
    method: SupportedVideoMethods;
    /**
     * If you're using this inside a content-overlay for example, you may want the video to play immediately on launch.
     * @type Boolean
     */
    autoplay: boolean;
    /**
     * Sets whether or not an autoplaying video should be muted.
     * By default autoplay videos will be muted.
     * @type Boolean
     */
    autoplayMute: boolean;
    /**
     * Supports YouTube and self hosted URLs.
     * @type String
     */
    src: string;
    /**
     * @type Number
     */
    width: number;
    /**
     * @type Number
     */
    height: number;
    /**
     * @type String
     */
    type: string;
    /**
     * @type String
     */
    playButtonAriaLabel: string;
    /**
     * @type String
     * @responsive
     */
    classes: ResponsiveString;
    classString: any;
    /**
     * @type Boolean
     */
    controls: boolean;
    /**
     * @type String
     */
    preload: string;
    /**
     * Fit the container this video is placed in. When the video is used in an overlay or another place where the video will takeover the existing height, it is recommended to enable this option.
     * @responsive
     * @type String
     */
    fitContainer: ResponsiveString;
    isLoaded: boolean;
    hasPoster: boolean;
    isPlayed: boolean;
    computedVideoConfig(): {};
    initVideo(): void;
    componentWillLoad(): void;
    componentDidLoad(): void;
    onClickHandler(event: MouseEvent): boolean;
    render(): any;
}
//# sourceMappingURL=content-video.d.ts.map